@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -45,7 +45,11 @@
 #include "ui_authenticationdialog.h"
 
 HttpWindow::HttpWindow(QWidget *parent)
+#ifdef Q_WS_MAEMO_5
+    : QWidget(parent)
+#else
     : QDialog(parent)
+#endif
 {
 #ifndef QT_NO_OPENSSL
     urlLineEdit = new QLineEdit("https://qt.nokia.com/");
@@ -57,6 +61,7 @@ HttpWindow::HttpWindow(QWidget *parent)
     urlLabel->setBuddy(urlLineEdit);
     statusLabel = new QLabel(tr("Please enter the URL of a file you want to "
                                 "download."));
+    statusLabel->setWordWrap(true);
 
     downloadButton = new QPushButton(tr("Download"));
     downloadButton->setDefault(true);
@@ -67,7 +72,9 @@ HttpWindow::HttpWindow(QWidget *parent)
     buttonBox->addButton(downloadButton, QDialogButtonBox::ActionRole);
     buttonBox->addButton(quitButton, QDialogButtonBox::RejectRole);
 
+#ifndef Q_WS_MAEMO_5
     progressDialog = new QProgressDialog(this);
+#endif
 
     connect(urlLineEdit, SIGNAL(textChanged(QString)),
             this, SLOT(enableDownloadButton()));
@@ -78,7 +85,9 @@ HttpWindow::HttpWindow(QWidget *parent)
     connect(&qnam, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)),
             this, SLOT(sslErrors(QNetworkReply*,QList<QSslError>)));
 #endif
+#ifndef Q_WS_MAEMO_5
     connect(progressDialog, SIGNAL(canceled()), this, SLOT(cancelDownload()));
+#endif
     connect(downloadButton, SIGNAL(clicked()), this, SLOT(downloadFile()));
     connect(quitButton, SIGNAL(clicked()), this, SLOT(close()));
 
@@ -117,7 +126,7 @@ void HttpWindow::downloadFile()
         fileName = "index.html";
 
     if (QFile::exists(fileName)) {
-        if (QMessageBox::question(this, tr("HTTP"), 
+        if (QMessageBox::question(this, tr("HTTP"),
                                   tr("There already exists a file called %1 in "
                                      "the current directory. Overwrite?").arg(fileName),
                                   QMessageBox::Yes|QMessageBox::No, QMessageBox::No)
@@ -136,9 +145,10 @@ void HttpWindow::downloadFile()
         return;
     }
 
-
+#ifndef Q_WS_MAEMO_5
     progressDialog->setWindowTitle(tr("HTTP"));
     progressDialog->setLabelText(tr("Downloading %1.").arg(fileName));
+#endif
     downloadButton->setEnabled(false);
 
     // schedule the request
@@ -164,11 +174,15 @@ void HttpWindow::httpFinished()
             file = 0;
         }
         reply->deleteLater();
+#ifndef Q_WS_MAEMO_5
         progressDialog->hide();
+#endif
         return;
     }
 
+#ifndef Q_WS_MAEMO_5
     progressDialog->hide();
+#endif
     file->flush();
     file->close();
 
@@ -194,7 +208,7 @@ void HttpWindow::httpFinished()
         }
     } else {
         QString fileName = QFileInfo(QUrl(urlLineEdit->text()).path()).fileName();
-        statusLabel->setText(tr("Downloaded %1 to current directory.").arg(fileName));
+        statusLabel->setText(tr("Downloaded %1 to %2.").arg(fileName).arg(QDir::currentPath()));
         downloadButton->setEnabled(true);
     }
 
@@ -219,8 +233,13 @@ void HttpWindow::updateDataReadProgress(qint64 bytesRead, qint64 totalBytes)
     if (httpRequestAborted)
         return;
 
+#ifndef Q_WS_MAEMO_5
     progressDialog->setMaximum(totalBytes);
     progressDialog->setValue(bytesRead);
+#else
+    Q_UNUSED(bytesRead);
+    Q_UNUSED(totalBytes);
+#endif
 }
 
 void HttpWindow::enableDownloadButton()
