@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -51,7 +51,6 @@
 #include <stdlib.h>
 
 //#define DEBUG_SOLUTION_GEN
-//#define DEBUG_PROJECT_GEN
 
 QT_BEGIN_NAMESPACE
 // Filter GUIDs (Do NOT change these!) ------------------------------
@@ -206,9 +205,19 @@ const char _slnExtSections[]    = "\n\tGlobalSection(ExtensibilityGlobals) = pos
                                   "\n\tEndGlobalSection";
 // ------------------------------------------------------------------
 
-VcprojGenerator::VcprojGenerator() : Win32MakefileGenerator(), init_flag(false)
+VcprojGenerator::VcprojGenerator()
+    : Win32MakefileGenerator(),
+      init_flag(false),
+      is64Bit(false),
+      projectWriter(0)
 {
 }
+
+VcprojGenerator::~VcprojGenerator()
+{
+    delete projectWriter;
+}
+
 bool VcprojGenerator::writeMakefile(QTextStream &t)
 {
     initProject(); // Fills the whole project with proper data
@@ -229,7 +238,7 @@ bool VcprojGenerator::writeMakefile(QTextStream &t)
         if(!project->isActiveConfig("build_pass")) {
             debug_msg(1, "Generator: MSVC.NET: Writing single configuration project file");
             XmlOutput xmlOut(t);
-            xmlOut << vcProject;
+            projectWriter->write(xmlOut, vcProject);
         }
         return true;
     }
@@ -272,7 +281,7 @@ bool VcprojGenerator::writeProjectMakefile()
            mergedProjects.at(0)->vcProject.Name ==
            mergedProjects.at(1)->vcProject.Name)
             mergedProjects.at(0)->writePrlFile();
-        mergedProject.Name = unescapeFilePath(project->first("QMAKE_ORIG_TARGET"));
+        mergedProject.Name = project->first("QMAKE_PROJECT_NAME");
         mergedProject.Version = mergedProjects.at(0)->vcProject.Version;
         mergedProject.ProjectGUID = project->isEmpty("QMAKE_UUID") ? getProjectUUID().toString().toUpper() : project->first("QMAKE_UUID");
         mergedProject.Keyword = project->first("VCPROJ_KEYWORD");
@@ -281,7 +290,7 @@ bool VcprojGenerator::writeProjectMakefile()
         mergedProject.PlatformName = mergedProjects.at(0)->vcProject.PlatformName;
 
         XmlOutput xmlOut(t);
-        xmlOut << mergedProject;
+        projectWriter->write(xmlOut, mergedProject);
         return true;
     } else if(project->first("TEMPLATE") == "vcsubdirs") {
         return writeMakefile(t);
@@ -352,6 +361,25 @@ QUuid VcprojGenerator::increaseUUID(const QUuid &id)
     return result;
 }
 
+QStringList VcprojGenerator::collectSubDirs(QMakeProject *proj)
+{
+    QStringList subdirs;
+    QStringList tmp_proj_subdirs = proj->variables()["SUBDIRS"];
+    for(int x = 0; x < tmp_proj_subdirs.size(); ++x) {
+        QString tmpdir = tmp_proj_subdirs.at(x);
+        if(!proj->isEmpty(tmpdir + ".file")) {
+            if(!proj->isEmpty(tmpdir + ".subdir"))
+                warn_msg(WarnLogic, "Cannot assign both file and subdir for subdir %s",
+                         tmpdir.toLatin1().constData());
+            tmpdir = proj->first(tmpdir + ".file");
+        } else if(!proj->isEmpty(tmpdir + ".subdir")) {
+            tmpdir = proj->first(tmpdir + ".subdir");
+        }
+        subdirs += tmpdir;
+    }
+    return subdirs;
+}
+
 void VcprojGenerator::writeSubDirs(QTextStream &t)
 {
     // Check if all requirements are fulfilled
@@ -386,7 +414,6 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
     QHash<QString, VcsolutionDepend*> solution_depends;
     QList<VcsolutionDepend*> solution_cleanup;
 
-    QStringList subdirs = project->values("SUBDIRS");
     QString oldpwd = qmake_getpwd();
 
     // Make sure that all temp projects are configured
@@ -395,16 +422,9 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
     QStringList old_after_vars = Option::after_user_vars;
     Option::after_user_vars.append("CONFIG+=release");
 
+    QStringList subdirs = collectSubDirs(project);
     for(int i = 0; i < subdirs.size(); ++i) {
         QString tmp = subdirs.at(i);
-        if(!project->isEmpty(tmp + ".file")) {
-            if(!project->isEmpty(tmp + ".subdir"))
-                warn_msg(WarnLogic, "Cannot assign both file and subdir for subdir %s",
-                         tmp.toLatin1().constData());
-            tmp = project->first(tmp + ".file");
-        } else if(!project->isEmpty(tmp + ".subdir")) {
-            tmp = project->first(tmp + ".subdir");
-        }
         QFileInfo fi(fileInfo(Option::fixPathToLocalOS(tmp, true)));
         if(fi.exists()) {
             if(fi.isDir()) {
@@ -428,19 +448,8 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
                         continue;
                     }
                     if(tmp_proj.first("TEMPLATE") == "vcsubdirs") {
-                        QStringList tmp_proj_subdirs = tmp_proj.variables()["SUBDIRS"];
-                        for(int x = 0; x < tmp_proj_subdirs.size(); ++x) {
-                            QString tmpdir = tmp_proj_subdirs.at(x);
-                            if(!tmp_proj.isEmpty(tmpdir + ".file")) {
-                                if(!tmp_proj.isEmpty(tmpdir + ".subdir"))
-                                    warn_msg(WarnLogic, "Cannot assign both file and subdir for subdir %s",
-                                            tmpdir.toLatin1().constData());
-                                tmpdir = tmp_proj.first(tmpdir + ".file");
-                            } else if(!tmp_proj.isEmpty(tmpdir + ".subdir")) {
-                                tmpdir = tmp_proj.first(tmpdir + ".subdir");
-                            }
+                        foreach(const QString &tmpdir, collectSubDirs(&tmp_proj))
                             subdirs += fileFixify(tmpdir);
-                        }
                     } else if(tmp_proj.first("TEMPLATE") == "vcapp" || tmp_proj.first("TEMPLATE") == "vclib") {
                         // Initialize a 'fake' project to get the correct variables
                         // and to be able to extract all the dependencies
@@ -451,6 +460,7 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
                         tmp_vcproj.setProjectFile(&tmp_proj);
                         Option::qmake_mode = old_mode;
                         if(Option::debug_level) {
+                            debug_msg(1, "Dumping all variables:");
                             QMap<QString, QStringList> &vars = tmp_proj.variables();
                             for(QMap<QString, QStringList>::Iterator it = vars.begin();
                                 it != vars.end(); ++it) {
@@ -460,8 +470,8 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
                             }
                         }
 
-                        // We assume project filename is [QMAKE_ORIG_TARGET].vcproj
-                        QString vcproj = unescapeFilePath(tmp_vcproj.project->first("QMAKE_ORIG_TARGET") + project->first("VCPROJ_EXTENSION"));
+                        // We assume project filename is [QMAKE_PROJECT_NAME].vcproj
+                        QString vcproj = unescapeFilePath(tmp_vcproj.project->first("QMAKE_PROJECT_NAME") + project->first("VCPROJ_EXTENSION"));
                         QString vcprojDir = qmake_getpwd();
 
                         // If file doesn't exsist, then maybe the users configuration
@@ -588,14 +598,16 @@ nextfile:
         }
     }
     t << _slnGlobalBeg;
+
+    QString slnConf = _slnSolutionConf;
     if (!project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH")) {
-        QString slnConfCE = _slnSolutionConf;
-        QString platform = QString("|") + project->values("CE_SDK").join(" ") + " (" + project->first("CE_ARCH") + ")";
-        slnConfCE.replace(QString("|Win32"), platform);
-        t << slnConfCE;
-    } else {
-        t << _slnSolutionConf;
+        QString slnPlatform = QString("|") + project->values("CE_SDK").join(" ") + " (" + project->first("CE_ARCH") + ")";
+        slnConf.replace(QString("|Win32"), slnPlatform);
+    } else if (is64Bit) {
+        slnConf.replace(QString("|Win32"), "|x64");
     }
+    t << slnConf;
+
     t << _slnProjDepBeg;
 
     // Restore previous after_user_var options
@@ -612,7 +624,7 @@ nextfile:
     t << _slnProjDepEnd;
     t << _slnProjConfBeg;
     for(QList<VcsolutionDepend*>::Iterator it = solution_cleanup.begin(); it != solution_cleanup.end(); ++it) {
-        QString platform = "Win32";
+        QString platform = is64Bit ? "x64" : "Win32";
         if (!project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH"))
             platform = project->values("CE_SDK").join(" ") + " (" + project->first("CE_ARCH") + ")";
         t << "\n\t\t" << (*it)->uuid << QString(_slnProjDbgConfTag1).arg(platform) << platform;
@@ -649,12 +661,14 @@ bool VcprojGenerator::hasBuiltinCompiler(const QString &file)
 
 void VcprojGenerator::init()
 {
-    if(init_flag)
+    if (init_flag)
         return;
-    if(project->first("TEMPLATE") == "vcsubdirs") { //too much work for subdirs
-        init_flag = true;
+    init_flag = true;
+    is64Bit = (project->first("QMAKE_TARGET.arch") == "x86_64");
+    projectWriter = createProjectWriter();
+
+    if(project->first("TEMPLATE") == "vcsubdirs") //too much work for subdirs
         return;
-    }
 
     debug_msg(1, "Generator: MSVC.NET: Initializing variables");
 
@@ -667,6 +681,20 @@ void VcprojGenerator::init()
         project->values("QMAKESPEC").append(qgetenv("QMAKESPEC"));
 
     processVars();
+
+    project->values("QMAKE_LIBS") += escapeFilePaths(project->values("LIBS"));
+    project->values("QMAKE_LIBS_PRIVATE") += escapeFilePaths(project->values("LIBS_PRIVATE"));
+
+    if(!project->values("VERSION").isEmpty()) {
+        QString version = project->values("VERSION")[0];
+        int firstDot = version.indexOf(".");
+        QString major = version.left(firstDot);
+        QString minor = version.right(version.length() - firstDot - 1);
+        minor.replace(QRegExp("\\."), "");
+        project->values("QMAKE_LFLAGS").append("/VERSION:" + major + "." + minor);
+    }
+
+    MakefileGenerator::init();
     initOld();           // Currently calling old DSP code to set variables. CLEAN UP!
 
     // Figure out what we're trying to build
@@ -749,11 +777,16 @@ void VcprojGenerator::init()
 
 bool VcprojGenerator::mergeBuildProject(MakefileGenerator *other)
 {
-    VcprojGenerator *otherVC = static_cast<VcprojGenerator*>(other);
-    if (!otherVC) {
+    if (!other || !other->projectFile()) {
+        warn_msg(WarnLogic, "VcprojGenerator: Cannot merge null project.");
+        return false;
+    }
+    if (other->projectFile()->first("MAKEFILE_GENERATOR") != project->first("MAKEFILE_GENERATOR")) {
         warn_msg(WarnLogic, "VcprojGenerator: Cannot merge other types of projects! (ignored)");
         return false;
     }
+
+    VcprojGenerator *otherVC = static_cast<VcprojGenerator*>(other);
     mergedProjects += otherVC;
     return true;
 }
@@ -777,6 +810,9 @@ void VcprojGenerator::initProject()
     // Own elements -----------------------------
     vcProject.Name = unescapeFilePath(project->first("QMAKE_ORIG_TARGET"));
     switch(which_dotnet_version()) {
+    case NET2010:
+        vcProject.Version = "10.00";
+        break;
     case NET2008:
         vcProject.Version = "9,00";
         break;
@@ -799,7 +835,7 @@ void VcprojGenerator::initProject()
 
     vcProject.Keyword = project->first("VCPROJ_KEYWORD");
     if (project->isEmpty("CE_SDK") || project->isEmpty("CE_ARCH")) {
-        vcProject.PlatformName = (vcProject.Configuration.idl.TargetEnvironment == midlTargetWin64 ? "Win64" : "Win32");
+        vcProject.PlatformName = (is64Bit ? "x64" : "Win32");
     } else {
         vcProject.PlatformName = project->values("CE_SDK").join(" ") + " (" + project->first("CE_ARCH") + ")";
     }
@@ -846,11 +882,24 @@ void VcprojGenerator::initConfiguration()
         break;
     }
 
+    conf.OutputDirectory = project->first("DESTDIR");
+    if (conf.OutputDirectory.isEmpty())
+        conf.OutputDirectory = ".\\";
+    if (!conf.OutputDirectory.endsWith("\\"))
+        conf.OutputDirectory += '\\';
+    if (conf.CompilerVersion >= NET2010) {
+        // The target name could have been changed.
+        conf.PrimaryOutput = project->first("TARGET");
+        if ( !conf.PrimaryOutput.isEmpty() && !project->first("TARGET_VERSION_EXT").isEmpty() && project->isActiveConfig("shared"))
+            conf.PrimaryOutput.append(project->first("TARGET_VERSION_EXT"));
+    }
+
     conf.Name = project->values("BUILD_NAME").join(" ");
     if (conf.Name.isEmpty())
         conf.Name = isDebug ? "Debug" : "Release";
+    conf.ConfigurationName = conf.Name;
     if (project->isEmpty("CE_SDK") || project->isEmpty("CE_ARCH")) {
-        conf.Name += (conf.idl.TargetEnvironment == midlTargetWin64 ? "|Win64" : "|Win32");
+        conf.Name += (is64Bit ? "|x64" : "|Win32");
     } else {
         conf.Name += "|" + project->values("CE_SDK").join(" ") + " (" + project->first("CE_ARCH") + ")";
     }
@@ -861,8 +910,6 @@ void VcprojGenerator::initConfiguration()
     conf.DeleteExtensionsOnClean = project->first("DeleteExtensionsOnClean");
     conf.ImportLibrary = conf.linker.ImportLibrary;
     conf.IntermediateDirectory = project->first("OBJECTS_DIR");
-    conf.OutputDirectory = ".";
-    conf.PrimaryOutput = project->first("PrimaryOutput");
     conf.WholeProgramOptimization = conf.compiler.WholeProgramOptimization;
     temp = project->first("UseOfATL");
     if(!temp.isEmpty())
@@ -887,7 +934,6 @@ void VcprojGenerator::initConfiguration()
     } else {
         conf.compiler.PreprocessorDefinitions += "NDEBUG";
     }
-
 }
 
 void VcprojGenerator::initCompilerTool()
@@ -907,39 +953,24 @@ void VcprojGenerator::initCompilerTool()
         conf.compiler.PrecompiledHeaderFile    = "$(IntDir)\\" + precompPch;
         conf.compiler.PrecompiledHeaderThrough = project->first("PRECOMPILED_HEADER");
         conf.compiler.ForcedIncludeFiles       = project->values("PRECOMPILED_HEADER");
-        // Minimal build option triggers an Internal Compiler Error
-        // when used in conjunction with /FI and /Yu, so remove it
-        project->values("QMAKE_CFLAGS_DEBUG").removeAll("-Gm");
-        project->values("QMAKE_CFLAGS_DEBUG").removeAll("/Gm");
-        project->values("QMAKE_CXXFLAGS_DEBUG").removeAll("-Gm");
-        project->values("QMAKE_CXXFLAGS_DEBUG").removeAll("/Gm");
+
+        if (conf.CompilerVersion <= NET2003) {
+            // Minimal build option triggers an Internal Compiler Error
+            // when used in conjunction with /FI and /Yu, so remove it
+            // ### work-around for a VS 2003 bug. Move to some prf file or remove completely.
+            project->values("QMAKE_CFLAGS_DEBUG").removeAll("-Gm");
+            project->values("QMAKE_CFLAGS_DEBUG").removeAll("/Gm");
+            project->values("QMAKE_CXXFLAGS_DEBUG").removeAll("-Gm");
+            project->values("QMAKE_CXXFLAGS_DEBUG").removeAll("/Gm");
+        }
     }
 
     conf.compiler.parseOptions(project->values("QMAKE_CXXFLAGS"));
-    if(project->isActiveConfig("debug")){
-        // Debug version
-        if((projectTarget == Application) || (projectTarget == StaticLib))
-            conf.compiler.parseOptions(project->values("QMAKE_CXXFLAGS_MT_DBG"));
-        else
-            conf.compiler.parseOptions(project->values("QMAKE_CXXFLAGS_MT_DLLDBG"));
-    } else {
-        // Release version
-        conf.compiler.PreprocessorDefinitions += "QT_NO_DEBUG";
-        conf.compiler.PreprocessorDefinitions += "NDEBUG";
-        if((projectTarget == Application) || (projectTarget == StaticLib))
-            conf.compiler.parseOptions(project->values("QMAKE_CXXFLAGS_MT"));
-        else
-            conf.compiler.parseOptions(project->values("QMAKE_CXXFLAGS_MT_DLL"));
-    }
 
-    // Common for both release and debug
-    if(project->isActiveConfig("windows"))
-        conf.compiler.PreprocessorDefinitions += project->values("MSVCPROJ_WINCONDEF");
-
-    // Can this be set for ALL configs?
-    // If so, use qmake.conf!
-    if(projectTarget == SharedLib)
+    if (project->isActiveConfig("windows"))
         conf.compiler.PreprocessorDefinitions += "_WINDOWS";
+    else if (project->isActiveConfig("console"))
+        conf.compiler.PreprocessorDefinitions += "_CONSOLE";
 
     conf.compiler.PreprocessorDefinitions += project->values("DEFINES");
     conf.compiler.PreprocessorDefinitions += project->values("PRL_EXPORT_DEFINES");
@@ -949,13 +980,7 @@ void VcprojGenerator::initCompilerTool()
 void VcprojGenerator::initLibrarianTool()
 {
     VCConfiguration &conf = vcProject.Configuration;
-    conf.librarian.OutputFile = project->first("DESTDIR");
-    if(conf.librarian.OutputFile.isEmpty())
-        conf.librarian.OutputFile = ".\\";
-
-    if(!conf.librarian.OutputFile.endsWith("\\"))
-        conf.librarian.OutputFile += '\\';
-
+    conf.librarian.OutputFile = "$(OutDir)\\";
     conf.librarian.OutputFile += project->first("MSVCPROJ_TARGET");
     conf.librarian.AdditionalOptions += project->values("QMAKE_LIBFLAGS");
 }
@@ -964,7 +989,17 @@ void VcprojGenerator::initLinkerTool()
 {
     findLibraries(); // Need to add the highest version of the libs
     VCConfiguration &conf = vcProject.Configuration;
-    conf.linker.parseOptions(project->values("MSVCPROJ_LFLAGS"));
+    conf.linker.parseOptions(project->values("QMAKE_LFLAGS"));
+
+    foreach (const QString &libDir, project->values("QMAKE_LIBDIR")) {
+        if (libDir.startsWith("/LIBPATH:"))
+            conf.linker.AdditionalLibraryDirectories += libDir.mid(9);
+        else
+            conf.linker.AdditionalLibraryDirectories += libDir;
+    }
+
+    if (!project->values("DEF_FILE").isEmpty())
+        conf.linker.ModuleDefinitionFile = project->first("DEF_FILE");
 
     foreach(QString libs, project->values("MSVCPROJ_LIBS")) {
         if (libs.left(9).toUpper() == "/LIBPATH:") {
@@ -975,24 +1010,7 @@ void VcprojGenerator::initLinkerTool()
         }
     }
 
-    switch (projectTarget) {
-    case Application:
-        conf.linker.OutputFile = project->first("DESTDIR");
-        break;
-    case SharedLib:
-        conf.linker.parseOptions(project->values("MSVCPROJ_LIBOPTIONS"));
-        conf.linker.OutputFile = project->first("DESTDIR");
-        break;
-    case StaticLib: //unhandled - added to remove warnings..
-        break;
-    }
-
-    if(conf.linker.OutputFile.isEmpty())
-        conf.linker.OutputFile = ".\\";
-
-    if(!conf.linker.OutputFile.endsWith("\\"))
-        conf.linker.OutputFile += '\\';
-
+    conf.linker.OutputFile = "$(OutDir)\\";
     conf.linker.OutputFile += project->first("MSVCPROJ_TARGET");
 
     if(project->isActiveConfig("dll")){
@@ -1011,10 +1029,9 @@ void VcprojGenerator::initResourceTool()
     if(project->isActiveConfig("debug"))
         conf.resource.PreprocessorDefinitions += "_DEBUG";
     if(project->isActiveConfig("staticlib"))
-        conf.resource.ResourceOutputFileName = project->first("DESTDIR") + "/$(InputName).res";
+        conf.resource.ResourceOutputFileName = "$(OutDir)\\$(InputName).res";
 }
 
-
 void VcprojGenerator::initIDLTool()
 {
 }
@@ -1030,22 +1047,26 @@ void VcprojGenerator::initPreBuildEventTools()
 void VcprojGenerator::initPostBuildEventTools()
 {
     VCConfiguration &conf = vcProject.Configuration;
-    if(!project->values("QMAKE_POST_LINK").isEmpty()) {
+    if (!project->values("QMAKE_POST_LINK").isEmpty()) {
         QStringList cmdline = VCToolBase::fixCommandLine(var("QMAKE_POST_LINK"));
         conf.postBuild.CommandLine = cmdline;
         conf.postBuild.Description = cmdline.join(QLatin1String("\r\n"));
+        conf.postBuild.ExcludedFromBuild = _False;
     }
 
     QString signature = !project->isEmpty("SIGNATURE_FILE") ? var("SIGNATURE_FILE") : var("DEFAULT_SIGNATURE");
     bool useSignature = !signature.isEmpty() && !project->isActiveConfig("staticlib") &&
                         !project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH");
-    if(useSignature)
+    if (useSignature) {
         conf.postBuild.CommandLine.prepend(
                 QLatin1String("signtool sign /F ") + signature + QLatin1String(" \"$(TargetPath)\""));
+        conf.postBuild.ExcludedFromBuild = _False;
+    }
 
-    if(!project->values("MSVCPROJ_COPY_DLL").isEmpty()) {
+    if (!project->values("MSVCPROJ_COPY_DLL").isEmpty()) {
         conf.postBuild.Description += var("MSVCPROJ_COPY_DLL_DESC");
         conf.postBuild.CommandLine += var("MSVCPROJ_COPY_DLL");
+        conf.postBuild.ExcludedFromBuild = _False;
     }
 }
 
@@ -1133,7 +1154,8 @@ void VcprojGenerator::initDeploymentTool()
             devicePath = Option::fixPathToLocalOS(QDir::cleanPath(targetPath + QLatin1Char('\\') + devicePath));
         }
         // foreach d in item.sources
-        foreach(QString source, project->values(item + ".sources")) {
+        // ### Qt 5: remove .sources, inconsistent with INSTALLS
+        foreach(QString source, project->values(item + ".sources") + project->values(item + ".files")) {
             QString itemDevicePath = devicePath;
             source = Option::fixPathToLocalOS(source);
             QString nameFilter;
@@ -1175,6 +1197,7 @@ void VcprojGenerator::initPreLinkEventTools()
         QStringList cmdline = VCToolBase::fixCommandLine(var("QMAKE_PRE_LINK"));
         conf.preLink.CommandLine = cmdline;
         conf.preLink.Description = cmdline.join(QLatin1String("\r\n"));
+        conf.preLink.ExcludedFromBuild = _False;
     }
 }
 
@@ -1266,7 +1289,6 @@ void VcprojGenerator::initTranslationFiles()
     vcProject.TranslationFiles.CustomBuild = none;
 }
 
-
 void VcprojGenerator::initFormFiles()
 {
     vcProject.FormFiles.Name = "Form Files";
@@ -1282,7 +1304,6 @@ void VcprojGenerator::initFormFiles()
     vcProject.FormFiles.CustomBuild = none;
 }
 
-
 void VcprojGenerator::initResourceFiles()
 {
     vcProject.ResourceFiles.Name = "Resource Files";
@@ -1302,6 +1323,9 @@ void VcprojGenerator::initResourceFiles()
 
                 dep_cmd = Option::fixPathToLocalOS(dep_cmd, true, false);
                 if(canExecute(dep_cmd)) {
+                    dep_cmd.prepend(QLatin1String("cd ")
+                                    + escapeFilePath(Option::fixPathToLocalOS(Option::output_dir, false))
+                                    + QLatin1String(" && "));
                     if(FILE *proc = QT_POPEN(dep_cmd.toLatin1().constData(), "r")) {
                         QString indeps;
                         while(!feof(proc)) {
@@ -1312,7 +1336,8 @@ void VcprojGenerator::initResourceFiles()
                         }
                         QT_PCLOSE(proc);
                         if(!indeps.isEmpty())
-                            deps += fileFixify(indeps.replace('\n', ' ').simplified().split(' '));
+                            deps += fileFixify(indeps.replace('\n', ' ').simplified().split(' '),
+                                               QString(), Option::output_dir);
                     }
                 }
             }
@@ -1410,86 +1435,11 @@ void VcprojGenerator::initExtraCompilerOutputs()
     }
 }
 
-/* \internal
-    Sets up all needed variables from the environment and all the different caches and .conf files
-*/
-
 void VcprojGenerator::initOld()
 {
-    if(init_flag)
-        return;
-
-    init_flag = true;
-    QStringList::Iterator it;
-
-    // Decode version, and add it to $$MSVCPROJ_VERSION --------------
-    if(!project->values("VERSION").isEmpty()) {
-        QString version = project->values("VERSION")[0];
-        int firstDot = version.indexOf(".");
-        QString major = version.left(firstDot);
-        QString minor = version.right(version.length() - firstDot - 1);
-        minor.replace(QRegExp("\\."), "");
-        project->values("MSVCPROJ_VERSION").append("/VERSION:" + major + "." + minor);
-        project->values("QMAKE_LFLAGS").append("/VERSION:" + major + "." + minor);
-    }
-
-    project->values("QMAKE_LIBS") += escapeFilePaths(project->values("LIBS"));
-    project->values("QMAKE_LIBS_PRIVATE") += escapeFilePaths(project->values("LIBS_PRIVATE"));
-
-     // Get filename w/o extension -----------------------------------
-    QString msvcproj_project = "";
-    QString targetfilename = "";
-    if(!project->isEmpty("TARGET")) {
-        project->values("TARGET") = unescapeFilePaths(project->values("TARGET"));
-        targetfilename = msvcproj_project = project->first("TARGET");
-    }
-
-    // Init base class too -------------------------------------------
-    MakefileGenerator::init();
-
-    if(msvcproj_project.isEmpty())
-        msvcproj_project = Option::output.fileName();
-
-    msvcproj_project = msvcproj_project.right(msvcproj_project.length() - msvcproj_project.lastIndexOf("\\") - 1);
-    msvcproj_project = msvcproj_project.left(msvcproj_project.lastIndexOf("."));
-    msvcproj_project.replace(QRegExp("-"), "");
-
-    project->values("MSVCPROJ_PROJECT").append(msvcproj_project);
-    QStringList &proj = project->values("MSVCPROJ_PROJECT");
-
-    for(it = proj.begin(); it != proj.end(); ++it)
-        (*it).replace(QRegExp("\\.[a-zA-Z0-9_]*$"), "");
-
-    // SUBSYSTEM -----------------------------------------------------
-    if(!project->values("QMAKE_APP_FLAG").isEmpty()) {
-            project->values("MSVCPROJ_TEMPLATE").append("win32app" + project->first("VCPROJ_EXTENSION"));
-            if(project->isActiveConfig("console")) {
-                project->values("MSVCPROJ_CONSOLE").append("CONSOLE");
-                project->values("MSVCPROJ_WINCONDEF").append("_CONSOLE");
-                project->values("MSVCPROJ_VCPROJTYPE").append("0x0103");
-                project->values("MSVCPROJ_SUBSYSTEM").append("CONSOLE");
-            } else {
-                project->values("MSVCPROJ_CONSOLE").clear();
-                project->values("MSVCPROJ_WINCONDEF").append("_WINDOWS");
-                project->values("MSVCPROJ_VCPROJTYPE").append("0x0101");
-                project->values("MSVCPROJ_SUBSYSTEM").append("WINDOWS");
-            }
-    }
-
     // $$QMAKE.. -> $$MSVCPROJ.. -------------------------------------
     project->values("MSVCPROJ_LIBS") += project->values("QMAKE_LIBS");
     project->values("MSVCPROJ_LIBS") += project->values("QMAKE_LIBS_PRIVATE");
-    project->values("MSVCPROJ_LFLAGS") += project->values("QMAKE_LFLAGS");
-    if(!project->values("QMAKE_LIBDIR").isEmpty()) {
-        QStringList strl = project->values("QMAKE_LIBDIR");
-        QStringList::iterator stri;
-        for(stri = strl.begin(); stri != strl.end(); ++stri) {
-            if(!(*stri).startsWith("/LIBPATH:"))
-                (*stri).prepend("/LIBPATH:");
-        }
-        project->values("MSVCPROJ_LFLAGS") += strl;
-    }
-    project->values("MSVCPROJ_CXXFLAGS") += project->values("QMAKE_CXXFLAGS");
     QStringList &incs = project->values("INCLUDEPATH");
     for(QStringList::Iterator incit = incs.begin(); incit != incs.end(); ++incit) {
         QString inc = (*incit);
@@ -1528,9 +1478,6 @@ void VcprojGenerator::initOld()
         project->values("MSVCPROJ_COPY_DLL_DESC").append(deststr);
     }
 
-    if (!project->values("DEF_FILE").isEmpty())
-        project->values("MSVCPROJ_LFLAGS").append("/DEF:"+project->first("DEF_FILE"));
-
     project->values("QMAKE_INTERNAL_PRL_LIBS") << "MSVCPROJ_LIBS";
 
     // Verbose output if "-d -d"...
@@ -1540,6 +1487,11 @@ void VcprojGenerator::initOld()
 // ------------------------------------------------------------------------------------------------
 // ------------------------------------------------------------------------------------------------
 
+VCProjectWriter *VcprojGenerator::createProjectWriter()
+{
+    return new VCProjectWriter;
+}
+
 QString VcprojGenerator::replaceExtraCompilerVariables(const QString &var, const QStringList &in, const QStringList &out)
 {
     QString ret = MakefileGenerator::replaceExtraCompilerVariables(var, in, out);
@@ -1558,8 +1510,6 @@ QString VcprojGenerator::replaceExtraCompilerVariables(const QString &var, const
     return ret;
 }
 
-
-
 bool VcprojGenerator::openOutput(QFile &file, const QString &/*build*/) const
 {
     QString outdir;
@@ -1611,28 +1561,13 @@ QString VcprojGenerator::findTemplate(QString file)
     return ret;
 }
 
-
-void VcprojGenerator::processPrlVariable(const QString &var, const QStringList &l)
-{
-    if(var == "QMAKE_PRL_DEFINES") {
-        QStringList &out = project->values("MSVCPROJ_DEFINES");
-        for(QStringList::ConstIterator it = l.begin(); it != l.end(); ++it) {
-            if(out.indexOf((*it)) == -1)
-                out.append((" /D " + *it));
-        }
-    } else {
-        MakefileGenerator::processPrlVariable(var, l);
-    }
-}
-
 void VcprojGenerator::outputVariables()
 {
 #if 0
     qDebug("Generator: MSVC.NET: List of current variables:");
     for(QMap<QString, QStringList>::ConstIterator it = project->variables().begin(); it != project->variables().end(); ++it)
-        qDebug("Generator: MSVC.NET: %s => %s", it.key().toLatin1().constData(), it.data().join(" | ").toLatin1().constData());
+        qDebug("Generator: MSVC.NET: %s => %s", qPrintable(it.key()), qPrintable(it.value().join(" | ")));
 #endif
 }
 
 QT_END_NAMESPACE
-
