@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -173,6 +173,7 @@ QString QIconvCodec::convertToUnicode(const char* chars, int len, ConverterState
     int invalidCount = 0;
     int remainingCount = 0;
     char *remainingBuffer = 0;
+    IconvState *temporaryState = 0;
     IconvState **pstate;
 
     if (convState) {
@@ -193,11 +194,11 @@ QString QIconvCodec::convertToUnicode(const char* chars, int len, ConverterState
             // we're running after the Q_GLOBAL_STATIC has been deleted
             // or before the QCoreApplication initialization
             // bad programmer, no cookie for you
-            return QString::fromLatin1(chars, len);
+            pstate = &temporaryState;
+        } else {
+            // stateless conversion -- use thread-local data
+            pstate = &toUnicodeState()->localData();
         }
-
-        // stateless conversion -- use thread-local data
-        pstate = &toUnicodeState()->localData();
     }
 
     if (!*pstate) {
@@ -280,20 +281,26 @@ QString QIconvCodec::convertToUnicode(const char* chars, int len, ConverterState
                 iconv(state->cd, 0, &inBytesLeft, 0, &outBytesLeft);
             }
 
+            delete temporaryState;
             return QString::fromLatin1(chars, len);
         }
     } while (inBytesLeft != 0);
 
-    QString s = utf16Codec->toUnicode(ba.constData(), ba.size() - outBytesLeft);
+    QString s;
 
     if (convState) {
+        s = utf16Codec->toUnicode(ba.constData(), ba.size() - outBytesLeft, &state->internalState);
+
         convState->invalidChars = invalidCount;
         convState->remainingChars = remainingCount;
     } else {
+        s = utf16Codec->toUnicode(ba.constData(), ba.size() - outBytesLeft);
+
         // reset state
         iconv(state->cd, 0, &inBytesLeft, 0, &outBytesLeft);
     }
 
+    delete temporaryState;
     return s;
 }
 
@@ -337,30 +344,22 @@ QByteArray QIconvCodec::convertFromUnicode(const QChar *uc, int len, ConverterSt
     char **inBytesPtr = &inBytes;
 #endif
 
+    IconvState *temporaryState = 0;
     QThreadStorage<QIconvCodec::IconvState *> *ts = fromUnicodeState();
-    if (!qt_locale_initialized || !ts) {
-        // we're running after the Q_GLOBAL_STATIC has been deleted
-        // or before the QCoreApplication initialization
-        // bad programmer, no cookie for you
-        if (!len)
-            // this is a special case - zero-sized string should be
-            // translated to empty but not-null QByteArray.
-            return QByteArray("");
-        return QString::fromRawData(uc, len).toLatin1();
-    }
-    IconvState *&state = ts->localData();
+    IconvState *&state = (qt_locale_initialized && ts) ? ts->localData() : temporaryState;
     if (!state) {
-        state = new IconvState(QIconvCodec::createIconv_t(0, UTF16));
-        if (state->cd == reinterpret_cast<iconv_t>(-1)) {
-            if (!setByteOrder(state->cd)) {
+        iconv_t cd = QIconvCodec::createIconv_t(0, UTF16);
+        if (cd != reinterpret_cast<iconv_t>(-1)) {
+            if (!setByteOrder(cd)) {
                 perror("QIconvCodec::convertFromUnicode: using Latin-1 for conversion, iconv failed for BOM");
 
-                iconv_close(state->cd);
-                state->cd = reinterpret_cast<iconv_t>(-1);
+                iconv_close(cd);
+                cd = reinterpret_cast<iconv_t>(-1);
 
                 return QString(uc, len).toLatin1();
             }
         }
+        state = new IconvState(cd);
     }
     if (state->cd == reinterpret_cast<iconv_t>(-1)) {
         static int reported = 0;
@@ -368,6 +367,7 @@ QByteArray QIconvCodec::convertFromUnicode(const QChar *uc, int len, ConverterSt
             fprintf(stderr,
                     "QIconvCodec::convertFromUnicode: using Latin-1 for conversion, iconv_open failed\n");
         }
+        delete temporaryState;
         return QString(uc, len).toLatin1();
     }
  
@@ -430,6 +430,7 @@ QByteArray QIconvCodec::convertFromUnicode(const QChar *uc, int len, ConverterSt
                     // reset to initial state
                     iconv(state->cd, 0, &inBytesLeft, 0, &outBytesLeft);
 
+                    delete temporaryState;
                     return QString(uc, len).toLatin1();
                 }
             }
@@ -445,6 +446,7 @@ QByteArray QIconvCodec::convertFromUnicode(const QChar *uc, int len, ConverterSt
     if (convState)
         convState->invalidChars = invalidCount;
 
+    delete temporaryState;
     return ba;
 }
 
@@ -464,8 +466,13 @@ iconv_t QIconvCodec::createIconv_t(const char *to, const char *from)
 
     iconv_t cd = (iconv_t) -1;
 #if defined(__GLIBC__) || defined(GNU_LIBICONV)
+#if defined(Q_OS_QNX)
+    // on QNX the default locale is UTF-8, and an empty string will cause iconv_open to fail
+    static const char empty_codeset[] = "UTF-8";
+#else
     // both GLIBC and libgnuiconv will use the locale's encoding if from or to is an empty string
     static const char empty_codeset[] = "";
+#endif
     const char *codeset = empty_codeset;
     cd = iconv_open(to ? to : codeset, from ? from : codeset);
 #else
