@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -71,6 +71,7 @@
 #  include <e32ldr.h>
 #  include "qeventdispatcher_symbian_p.h"
 #  include "private/qcore_symbian_p.h"
+#  include "private/qfilesystemengine_p.h"
 #  include <apacmdln.h>
 #elif defined(Q_OS_UNIX)
 #  if !defined(QT_NO_GLIB)
@@ -97,6 +98,12 @@
 #  include <taskLib.h>
 #endif
 
+#ifdef Q_OS_QNX
+#  include <sys/neutrino.h>
+#  include <pthread.h>
+#  include <sched.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 class QMutexUnlocker
@@ -115,8 +122,6 @@ private:
 };
 
 #ifdef Q_OS_SYMBIAN
-typedef TDriveNumber (*SystemDriveFunc)(RFs&);
-static SystemDriveFunc PtrGetSystemDrive = 0;
 static CApaCommandLine* apaCommandLine = 0;
 static char *apaTail = 0;
 static QVector<char *> *apaArgv = 0;
@@ -174,6 +179,11 @@ CApaCommandLine* QCoreApplicationPrivate::symbianCommandLine()
 extern QString qAppFileName();
 #endif
 
+int QCoreApplicationPrivate::app_compile_version = 0x040000; //we don't know exactly, but it's at least 4.0.0
+#if defined(QT3_SUPPORT)
+bool QCoreApplicationPrivate::useQt3Support = true;
+#endif
+
 #if !defined(Q_OS_WIN)
 #ifdef Q_OS_MAC
 QString QCoreApplicationPrivate::macMenuBarName()
@@ -263,6 +273,40 @@ bool QCoreApplicationPrivate::is_app_closing = false;
 // initialized in qcoreapplication and in qtextstream autotest when setlocale is called.
 Q_CORE_EXPORT bool qt_locale_initialized = false;
 
+#ifdef Q_OS_SYMBIAN
+// The global QSettings needs to be cleaned where cleanup stack is available, which is not
+// normally the case when global static destructors are run.
+// Declare a custom QGlobalStaticDeleter for QSettings to handle that case.
+template<>
+class QGlobalStaticDeleter<QSettings>
+{
+public:
+    QGlobalStatic<QSettings> &globalStatic;
+    QGlobalStaticDeleter(QGlobalStatic<QSettings> &_globalStatic)
+        : globalStatic(_globalStatic)
+    { }
+
+    inline ~QGlobalStaticDeleter()
+    {
+        CTrapCleanup *cleanup = CTrapCleanup::New();
+        delete globalStatic.pointer;
+        delete cleanup;
+        globalStatic.pointer = 0;
+        globalStatic.destroyed = true;
+    }
+};
+#endif
+
+/*
+  Create an instance of Trolltech.conf. This ensures that the settings will not
+  be thrown out of QSetting's cache for unused settings.
+  */
+Q_GLOBAL_STATIC_WITH_ARGS(QSettings, staticTrolltechConf, (QSettings::UserScope, QLatin1String("Trolltech")))
+
+QSettings *QCoreApplicationPrivate::trolltechConf()
+{
+    return staticTrolltechConf();
+}
 
 Q_CORE_EXPORT uint qGlobalPostedEventsCount()
 {
@@ -315,10 +359,14 @@ struct QCoreApplicationData {
 
 Q_GLOBAL_STATIC(QCoreApplicationData, coreappdata)
 
-QCoreApplicationPrivate::QCoreApplicationPrivate(int &aargc, char **aargv)
+QCoreApplicationPrivate::QCoreApplicationPrivate(int &aargc, char **aargv, uint flags)
     : QObjectPrivate(), argc(aargc), argv(aargv), application_type(0), eventFilter(0),
       in_exec(false), aboutToQuitEmitted(false)
 {
+    app_compile_version = flags & 0xffffff;
+#if defined(QT3_SUPPORT)
+    useQt3Support = !(flags & 0x01000000);
+#endif
     static const char *const empty = "";
     if (argc == 0 || argv == 0) {
         argc = 0;
@@ -334,6 +382,22 @@ QCoreApplicationPrivate::QCoreApplicationPrivate(int &aargc, char **aargv)
     qt_application_thread_id = QThread::currentThreadId();
 #endif
 
+#ifdef Q_OS_QNX
+    // make the kernel attempt to emulate an instruction with a misaligned access
+    // if the attempt fails, it faults with a SIGBUS
+    int tv = -1;
+    ThreadCtl(_NTO_TCTL_ALIGN_FAULT, &tv);
+
+    // without Round Robin drawn intensive apps will hog the cpu
+    // and make the system appear frozen
+    int sched_policy;
+    sched_param param;
+    if (pthread_getschedparam(0, &sched_policy, &param) == 0 && sched_policy != SCHED_RR) {
+        sched_policy = SCHED_RR;
+        pthread_setschedparam(0, sched_policy, &param);
+    }
+#endif
+
     // note: this call to QThread::currentThread() may end up setting theMainThread!
     if (QThread::currentThread() != theMainThread)
         qWarning("WARNING: QApplication was not created in the main() thread.");
@@ -566,7 +630,7 @@ void QCoreApplication::flush()
     one valid character string.
 */
 QCoreApplication::QCoreApplication(int &argc, char **argv)
-    : QObject(*new QCoreApplicationPrivate(argc, argv))
+    : QObject(*new QCoreApplicationPrivate(argc, argv, 0x040000))
 {
     init();
     QCoreApplicationPrivate::eventDispatcher->startingUp();
@@ -582,6 +646,25 @@ QCoreApplication::QCoreApplication(int &argc, char **argv)
 #endif
 }
 
+QCoreApplication::QCoreApplication(int &argc, char **argv, int _internal)
+: QObject(*new QCoreApplicationPrivate(argc, argv, _internal))
+{
+    init();
+    QCoreApplicationPrivate::eventDispatcher->startingUp();
+#if defined(Q_OS_SYMBIAN)
+#ifndef QT_NO_LIBRARY
+    // Refresh factoryloader, as text codecs are requested during lib path
+    // resolving process and won't be therefore properly loaded.
+    // Unknown if this is symbian specific issue.
+    QFactoryLoader::refreshAll();
+#endif
+#ifndef QT_NO_SYSTEMLOCALE
+    d_func()->symbianInit();
+#endif
+#endif //Q_OS_SYMBIAN
+}
+
+
 // ### move to QCoreApplicationPrivate constructor?
 void QCoreApplication::init()
 {
@@ -595,6 +678,12 @@ void QCoreApplication::init()
     Q_ASSERT_X(!self, "QCoreApplication", "there should be only one application object");
     QCoreApplication::self = this;
 
+#ifdef Q_OS_SYMBIAN
+    //ensure temp and working directories exist
+    QFileSystemEngine::createDirectory(QFileSystemEntry(QFileSystemEngine::tempPath()), true);
+    QFileSystemEngine::createDirectory(QFileSystemEntry(QFileSystemEngine::currentPath()), true);
+#endif
+
 #ifndef QT_NO_THREAD
     QThread::initialize();
 #endif
@@ -860,11 +949,6 @@ bool QCoreApplication::notify(QObject *receiver, QEvent *event)
     d->checkReceiverThread(receiver);
 #endif
 
-#ifdef QT3_SUPPORT
-    if (event->type() == QEvent::ChildRemoved && !receiver->d_func()->pendingChildInsertedEvents.isEmpty())
-        receiver->d_func()->removePendingChildInsertedEvents(static_cast<QChildEvent *>(event)->child());
-#endif // QT3_SUPPORT
-
     return receiver->isWidgetType() ? false : d->notify_helper(receiver, event);
 }
 
@@ -1074,6 +1158,7 @@ int QCoreApplication::exec()
     return returnCode;
 }
 
+
 /*!
   Tells the application to exit with a return code.
 
@@ -1226,20 +1311,7 @@ void QCoreApplication::postEvent(QObject *receiver, QEvent *event, int priority)
     // delete the event on exceptions to protect against memory leaks till the event is
     // properly owned in the postEventList
     QScopedPointer<QEvent> eventDeleter(event);
-    if (data->postEventList.isEmpty() || data->postEventList.last().priority >= priority) {
-        // optimization: we can simply append if the last event in
-        // the queue has higher or equal priority
-        data->postEventList.append(QPostEvent(receiver, event, priority));
-    } else {
-        // insert event in descending priority order, using upper
-        // bound for a given priority (to ensure proper ordering
-        // of events with the same priority)
-        QPostEventList::iterator begin = data->postEventList.begin()
-                                         + data->postEventList.insertionOffset,
-                                   end = data->postEventList.end();
-        QPostEventList::iterator at = qUpperBound(begin, end, priority);
-        data->postEventList.insert(at, QPostEvent(receiver, event, priority));
-    }
+    data->postEventList.addEvent(QPostEvent(receiver, event, priority));
     eventDeleter.take();
     event->posted = true;
     ++receiver->d_func()->postedEvents;
@@ -1399,7 +1471,7 @@ void QCoreApplicationPrivate::sendPostedEvents(QObject *receiver, int event_type
                 // cannot send deferred delete
                 if (!event_type && !receiver) {
                     // don't lose the event
-                    data->postEventList.append(pe);
+                    data->postEventList.addEvent(pe);
                     const_cast<QPostEvent &>(pe).event = 0;
                 }
                 continue;
@@ -1530,7 +1602,7 @@ void QCoreApplication::removePostedEvents(QObject *receiver, int eventType)
             --pe.receiver->d_func()->postedEvents;
 #ifdef QT3_SUPPORT
             if (pe.event->type() == QEvent::ChildInsertedRequest)
-                pe.receiver->d_func()->removePendingChildInsertedEvents(0);
+                pe.receiver->d_func()->pendingChildInsertedEvents.clear();
 #endif
             pe.event->posted = false;
             events.append(pe.event);
@@ -1908,10 +1980,7 @@ QString QCoreApplication::applicationDirPath()
         }
         if (err != KErrNone || (driveInfo.iDriveAtt & KDriveAttRom) || (driveInfo.iMediaAtt
             & KMediaAttWriteProtected)) {
-            if(!PtrGetSystemDrive)
-                PtrGetSystemDrive = reinterpret_cast<SystemDriveFunc>(qt_resolveS60PluginFunc(S60Plugin_GetSystemDrive));
-            Q_ASSERT(PtrGetSystemDrive);
-            drive = PtrGetSystemDrive(fs);
+            drive = fs.GetSystemDrive();
             fs.DriveToChar(drive, driveChar);
         }
 
@@ -1922,12 +1991,6 @@ QString QCoreApplication::applicationDirPath()
         appPath = qt_TDesC2QString(privatePath);
         appPath.prepend(QLatin1Char(':')).prepend(qDriveChar);
 
-        // Create the appPath if it doesn't exist. Non-existing appPath will cause
-        // Platform Security violations later on if the app doesn't have AllFiles capability.
-        err = fs.CreatePrivatePath(drive);
-        if (err != KErrNone)
-            qWarning("QCoreApplication::applicationDirPath: Failed to create private path.");
-
         d->cachedApplicationDirPath = QFileInfo(appPath).path();
     }
 #else
@@ -2266,6 +2329,33 @@ QString QCoreApplication::applicationVersion()
 
 #ifndef QT_NO_LIBRARY
 
+#if defined(Q_OS_SYMBIAN)
+void qt_symbian_installLibraryPaths(QString installPathPlugins, QStringList& libPaths)
+{
+    // Add existing path on all drives for relative PluginsPath in Symbian
+    QString tempPath = installPathPlugins;
+    if (tempPath.at(tempPath.length() - 1) != QDir::separator()) {
+        tempPath += QDir::separator();
+    }
+    RFs& fs = qt_s60GetRFs();
+    TPtrC tempPathPtr(reinterpret_cast<const TText*> (tempPath.constData()));
+    // Symbian searches should start from Y:. Fix start drive otherwise TFindFile starts from the session drive
+    _LIT(KStartDir, "Y:");
+    TFileName dirPath(KStartDir);
+    dirPath.Append(tempPathPtr);
+    TFindFile finder(fs);
+    TInt err = finder.FindByDir(tempPathPtr, dirPath);
+    while (err == KErrNone) {
+        QString foundDir(reinterpret_cast<const QChar *>(finder.File().Ptr()),
+                         finder.File().Length());
+        foundDir = QDir(foundDir).canonicalPath();
+        if (!libPaths.contains(foundDir))
+            libPaths.append(foundDir);
+        err = finder.Find();
+    }
+}
+#endif
+
 Q_GLOBAL_STATIC_WITH_ARGS(QMutex, libraryPathMutex, (QMutex::Recursive))
 
 /*!
@@ -2298,24 +2388,8 @@ QStringList QCoreApplication::libraryPaths()
         QStringList *app_libpaths = coreappdata()->app_libpaths = new QStringList;
         QString installPathPlugins =  QLibraryInfo::location(QLibraryInfo::PluginsPath);
 #if defined(Q_OS_SYMBIAN)
-        // Add existing path on all drives for relative PluginsPath in Symbian
         if (installPathPlugins.at(1) != QChar(QLatin1Char(':'))) {
-            QString tempPath = installPathPlugins;
-            if (tempPath.at(tempPath.length() - 1) != QDir::separator()) {
-                tempPath += QDir::separator();
-            }
-            RFs& fs = qt_s60GetRFs();
-            TPtrC tempPathPtr(reinterpret_cast<const TText*> (tempPath.constData()));
-            TFindFile finder(fs);
-            TInt err = finder.FindByDir(tempPathPtr, tempPathPtr);
-            while (err == KErrNone) {
-                QString foundDir(reinterpret_cast<const QChar *>(finder.File().Ptr()),
-                                 finder.File().Length());
-                foundDir = QDir(foundDir).canonicalPath();
-                if (!app_libpaths->contains(foundDir))
-                    app_libpaths->append(foundDir);
-                err = finder.Find();
-            }
+            qt_symbian_installLibraryPaths(installPathPlugins, *app_libpaths);
         }
 #else
         if (QFile::exists(installPathPlugins)) {
@@ -2430,6 +2504,36 @@ void QCoreApplication::removeLibraryPath(const QString &path)
     QFactoryLoader::refreshAll();
 }
 
+#if defined(Q_OS_SYMBIAN)
+void QCoreApplicationPrivate::rebuildInstallLibraryPaths()
+{
+    // check there is not a single fixed install path
+    QString nativeInstallPathPlugins =  QLibraryInfo::location(QLibraryInfo::PluginsPath);
+    if (nativeInstallPathPlugins.at(1) == QChar(QLatin1Char(':')))
+        return;
+    QString installPathPlugins = QDir::cleanPath(nativeInstallPathPlugins);
+    // look for the install path at the drive roots
+    installPathPlugins.prepend(QChar(QLatin1Char(':')));
+
+    QMutexLocker locker(libraryPathMutex());
+    QStringList &app_libpaths = *coreappdata()->app_libpaths;
+    // Build a new library path, copying non-installPath components, and replacing existing install path with new
+    QStringList newPaths;
+    bool installPathFound = false;
+    foreach (QString path, app_libpaths) {
+        if (path.mid(1).compare(installPathPlugins, Qt::CaseInsensitive) == 0) {
+            // skip existing install paths, insert new install path when we find the first
+            if (!installPathFound)
+                qt_symbian_installLibraryPaths(nativeInstallPathPlugins, newPaths);
+            installPathFound = true;
+        } else {
+            newPaths.append(path);
+        }
+    }
+    app_libpaths = newPaths;
+}
+#endif
+
 #endif //QT_NO_LIBRARY
 
 /*!
