@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -218,12 +218,20 @@ QObject *QMetaObject::newInstance(QGenericArgument val0,
 */
 int QMetaObject::static_metacall(Call cl, int idx, void **argv) const
 {
-    if (priv(d.data)->revision < 2)
-        return 0;
-    const QMetaObjectExtraData *extra = (const QMetaObjectExtraData*)(d.extradata);
-    if (!extra || !extra->static_metacall)
-        return 0;
-    return extra->static_metacall(cl, idx, argv);
+    const QMetaObjectExtraData *extra = reinterpret_cast<const QMetaObjectExtraData *>(d.extradata);
+    if (priv(d.data)->revision >= 6) {
+        if (!extra || !extra->static_metacall)
+            return 0;
+        extra->static_metacall(0, cl, idx, argv);
+        return -1;
+    } else if (priv(d.data)->revision >= 2) {
+        if (!extra || !extra->static_metacall)
+            return 0;
+        typedef int (*OldMetacall)(QMetaObject::Call, int, void **);
+        OldMetacall o = reinterpret_cast<OldMetacall>(extra->static_metacall);
+        return o(cl, idx, argv);
+    }
+    return 0;
 }
 
 /*!
@@ -266,7 +274,25 @@ QObject *QMetaObject::cast(QObject *obj) const
         const QMetaObject *m = obj->metaObject();
         do {
             if (m == this)
-                return const_cast<QObject*>(obj);
+                return obj;
+        } while ((m = m->d.superdata));
+    }
+    return 0;
+}
+
+/*!
+    \internal
+
+    Returns \a obj if object \a obj inherits from this
+    meta-object; otherwise returns 0.
+*/
+const QObject *QMetaObject::cast(const QObject *obj) const
+{
+    if (obj) {
+        const QMetaObject *m = obj->metaObject();
+        do {
+            if (m == this)
+                return obj;
         } while ((m = m->d.superdata));
     }
     return 0;
@@ -406,9 +432,10 @@ int QMetaObject::constructorCount() const
 }
 
 /*!
-    Returns the number of methods in this class, including the number of
-    properties provided by each base class. These include signals and slots
-    as well as normal member functions.
+    Returns the number of methods known to the meta-object system in this class,
+    including the number of properties provided by each base class. These
+    include signals and slots as well as member functions declared with the
+    Q_INVOKABLE macro.
 
     Use code like the following to obtain a QStringList containing the methods
     specific to a given class:
@@ -621,22 +648,23 @@ int QMetaObjectPrivate::indexOfSignalRelative(const QMetaObject **baseObject,
 */
 int QMetaObject::indexOfSlot(const char *slot) const
 {
-    int i = QMetaObjectPrivate::indexOfSlot(this, slot, false);
+    const QMetaObject *m = this;
+    int i = QMetaObjectPrivate::indexOfSlotRelative(&m, slot, false);
     if (i < 0)
-        i = QMetaObjectPrivate::indexOfSlot(this, slot, true);
-    return i;
-}
-
-int QMetaObjectPrivate::indexOfSlot(const QMetaObject *m,
-                                    const char *slot,
-                                    bool normalizeStringData)
-{
-    int i = indexOfMethodRelative<MethodSlot>(&m, slot, normalizeStringData);
+        i = QMetaObjectPrivate::indexOfSlotRelative(&m, slot, true);
     if (i >= 0)
         i += m->methodOffset();
     return i;
 }
 
+// same as indexOfSignalRelative but for slots.
+int QMetaObjectPrivate::indexOfSlotRelative(const QMetaObject **m,
+                                    const char *slot,
+                                    bool normalizeStringData)
+{
+    return indexOfMethodRelative<MethodSlot>(m, slot, normalizeStringData);
+}
+
 static const QMetaObject *QMetaObject_findMetaObject(const QMetaObject *self, const char *name)
 {
     while (self) {
@@ -1218,6 +1246,10 @@ bool QMetaObject::invokeMethod(QObject *obj,
     tag(), and an access() specifier. You can use invoke() to invoke
     the method on an arbitrary QObject.
 
+    A method will only be registered with the meta-object system if it
+    is a slot, a signal, or declared with the Q_INVOKABLE macro.
+    Constructors can also be registered with Q_INVOKABLE.
+
     \sa QMetaObject, QMetaEnum, QMetaProperty, {Qt's Property System}
 */
 
@@ -1337,7 +1369,30 @@ const char *QMetaMethod::typeName() const
     Returns the tag associated with this method.
 
     Tags are special macros recognized by \c moc that make it
-    possible to add extra information about a method. For the moment,
+    possible to add extra information about a method.
+
+    Tag information can be added in the following
+    way in the function declaration:
+
+    \code
+        #define THISISTESTTAG // tag text
+        ...
+        private slots:
+            THISISTESTTAG void testFunc();
+    \endcode
+
+    and the information can be accessed by using:
+
+    \code
+        MainWindow win;
+        win.show();
+
+        int functionIndex = win.metaObject()->indexOfSlot("testFunc()");
+        QMetaMethod mm = metaObject()->method(functionIndex);
+        qDebug() << mm.tag(); // prints THISISTESTTAG
+    \endcode
+
+    For the moment,
     \c moc doesn't support any special tags.
 */
 const char *QMetaMethod::tag() const
@@ -1368,6 +1423,25 @@ int QMetaMethod::methodIndex() const
     return ((handle - priv(mobj->d.data)->methodData) / 5) + mobj->methodOffset();
 }
 
+/*!
+    \internal
+
+    Returns the method revision if one was
+    specified by Q_REVISION, otherwise returns 0.
+ */
+int QMetaMethod::revision() const
+{
+    if (!mobj)
+        return 0;
+    if ((QMetaMethod::Access)(mobj->d.data[handle + 4] & MethodRevisioned)) {
+        int offset = priv(mobj->d.data)->methodData
+                     + priv(mobj->d.data)->methodCount * 5
+                     + (handle - priv(mobj->d.data)->methodData) / 5;
+        return mobj->d.data[offset];
+    }
+    return 0;
+}
+
 /*!
     Returns the access specification of this method (private,
     protected, or public).
@@ -1558,6 +1632,12 @@ bool QMetaMethod::invoke(QObject *object,
                          : Qt::QueuedConnection;
     }
 
+#ifdef QT_NO_THREAD
+    if (connectionType == Qt::BlockingQueuedConnection) {
+        connectionType = Qt::DirectConnection;
+    }
+#endif
+
     // invoke!
     void *param[] = {
         returnValue.data(),
@@ -1573,10 +1653,20 @@ bool QMetaMethod::invoke(QObject *object,
         val9.data()
     };
     // recompute the methodIndex by reversing the arithmetic in QMetaObject::property()
-    int methodIndex = ((handle - priv(mobj->d.data)->methodData) / 5) + mobj->methodOffset();
+    int idx_relative = ((handle - priv(mobj->d.data)->methodData) / 5);
+    int idx_offset =  mobj->methodOffset();
+    QObjectPrivate::StaticMetaCallFunction callFunction =
+        (QMetaObjectPrivate::get(mobj)->revision >= 6 && mobj->d.extradata)
+        ? reinterpret_cast<const QMetaObjectExtraData *>(mobj->d.extradata)->static_metacall : 0;
+
     if (connectionType == Qt::DirectConnection) {
-        return QMetaObject::metacall(object, QMetaObject::InvokeMetaMethod, methodIndex, param) < 0;
-    } else {
+        if (callFunction) {
+            callFunction(object, QMetaObject::InvokeMetaMethod, idx_relative, param);
+            return true;
+        } else {
+            return QMetaObject::metacall(object, QMetaObject::InvokeMetaMethod, idx_relative + idx_offset, param) < 0;
+        }
+    } else if (connectionType == Qt::QueuedConnection) {
         if (returnValue.data()) {
             qWarning("QMetaMethod::invoke: Unable to invoke methods with return values in "
                      "queued connections");
@@ -1609,40 +1699,21 @@ bool QMetaMethod::invoke(QObject *object,
             }
         }
 
-        if (connectionType == Qt::QueuedConnection) {
-            QCoreApplication::postEvent(object, new QMetaCallEvent(methodIndex,
-                                                                   0,
-                                                                   -1,
-                                                                   nargs,
-                                                                   types,
-                                                                   args));
-        } else {
-            if (currentThread == objectThread) {
-                qWarning("QMetaMethod::invoke: Dead lock detected in "
-                         "BlockingQueuedConnection: Receiver is %s(%p)",
-                         mobj->className(), object);
-            }
-
-            // blocking queued connection
-#ifdef QT_NO_THREAD
-            QCoreApplication::postEvent(object, new QMetaCallEvent(methodIndex,
-                                                                   0,
-                                                                   -1,
-                                                                   nargs,
-                                                                   types,
-                                                                   args));
-#else
-            QSemaphore semaphore;
-            QCoreApplication::postEvent(object, new QMetaCallEvent(methodIndex,
-                                                                   0,
-                                                                   -1,
-                                                                   nargs,
-                                                                   types,
-                                                                   args,
-                                                                   &semaphore));
-            semaphore.acquire();
-#endif // QT_NO_THREAD
+        QCoreApplication::postEvent(object, new QMetaCallEvent(idx_offset, idx_relative, callFunction,
+                                                        0, -1, nargs, types, args));
+    } else { // blocking queued connection
+#ifndef QT_NO_THREAD
+        if (currentThread == objectThread) {
+            qWarning("QMetaMethod::invoke: Dead lock detected in "
+                        "BlockingQueuedConnection: Receiver is %s(%p)",
+                        mobj->className(), object);
         }
+
+        QSemaphore semaphore;
+        QCoreApplication::postEvent(object, new QMetaCallEvent(idx_offset, idx_relative, callFunction,
+                                                        0, -1, 0, 0, param, &semaphore));
+        semaphore.acquire();
+#endif // QT_NO_THREAD
     }
     return true;
 }
@@ -2388,6 +2459,35 @@ int QMetaProperty::notifySignalIndex() const
     }
 }
 
+/*!
+    \internal
+
+    Returns the property revision if one was
+    specified by REVISION, otherwise returns 0.
+ */
+int QMetaProperty::revision() const
+{
+    if (!mobj)
+        return 0;
+    int flags = mobj->d.data[handle + 2];
+    if (flags & Revisioned) {
+        int offset = priv(mobj->d.data)->propertyData +
+                     priv(mobj->d.data)->propertyCount * 3 + idx;
+        // Revision data is placed after NOTIFY data, if present.
+        // Iterate through properties to discover whether we have NOTIFY signals.
+        for (int i = 0; i < priv(mobj->d.data)->propertyCount; ++i) {
+            int handle = priv(mobj->d.data)->propertyData + 3*i;
+            if (mobj->d.data[handle + 2] & Notify) {
+                offset += priv(mobj->d.data)->propertyCount;
+                break;
+            }
+        }
+        return mobj->d.data[offset];
+    } else {
+        return 0;
+    }
+}
+
 /*!
     Returns true if this property is writable; otherwise returns
     false.
