@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -131,7 +131,7 @@ QDeclarativeCustomParserNodePrivate::fromProperty(QDeclarativeParser::Property *
             prop.d->values << QVariant::fromValue(props.at(ii));
     } else {
         for(int ii = 0; ii < p->values.count(); ++ii) {
-            Value *v = p->values.at(ii);
+            QDeclarativeParser::Value *v = p->values.at(ii);
             v->type = QDeclarativeParser::Value::Literal;
 
             if(v->object) {
