@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -704,13 +704,16 @@ int QAccessibleWidget::navigate(RelationFlag relation, int entry,
             int sibCount = pIface->childCount();
             QAccessibleInterface *candidate = 0;
             for (int i = 0; i < sibCount && entry; ++i) {
-                pIface->navigate(Child, i+1, &candidate);
-                Q_ASSERT(candidate);
-                if (candidate->relationTo(0, this, 0) & Label)
+                const int childId = pIface->navigate(Child, i+1, &candidate);
+                Q_ASSERT(childId >= 0);
+                if (childId > 0)
+                    candidate = pIface;
+                if (candidate->relationTo(childId, this, 0) & Label)
                     --entry;
                 if (!entry)
                     break;
-                delete candidate;
+                if (candidate != pIface)
+                    delete candidate;
                 candidate = 0;
             }
             if (!candidate) {
@@ -885,12 +888,15 @@ bool QAccessibleWidget::doAction(int action, int child, const QVariantList &para
     if (action == SetFocus || action == DefaultAction) {
         if (child || !widget()->isEnabled())
             return false;
-        if (widget()->focusPolicy() != Qt::NoFocus)
-            widget()->setFocus();
-        else if (widget()->isWindow())
-            widget()->activateWindow();
-        else
+
+        if ((widget()->focusPolicy() == Qt::NoFocus) && (!widget()->isWindow()))
             return false;
+
+        if (!widget()->isWindow())
+            widget()->setFocus();
+
+        widget()->activateWindow();
+
         return true;
     } else if (action > 0) {
         if (QAction *act = widget()->actions().value(action - 1)) {
@@ -934,7 +940,7 @@ QAccessible::State QAccessibleWidget::state(int child) const
     QWidget *w = widget();
     if (w->testAttribute(Qt::WA_WState_Visible) == false)
         state |= Invisible;
-    if (w->focusPolicy() != Qt::NoFocus && w->isActiveWindow())
+    if (w->focusPolicy() != Qt::NoFocus)
         state |= Focusable;
     if (w->hasFocus())
         state |= Focused;
@@ -1015,7 +1021,7 @@ QVariant QAccessibleWidgetEx::invokeMethodEx(Method method, int child, const QVa
     case ListSupportedMethods: {
         QSet<QAccessible::Method> set;
         set << ListSupportedMethods << ForegroundColor << BackgroundColor;
-        return qVariantFromValue(set);
+        return QVariant::fromValue(set);
     }
     case ForegroundColor:
         return widget()->palette().color(widget()->foregroundRole());
