@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,6 +42,7 @@
 #include "qplatformdefs.h"
 
 #include "qapplication.h"
+#include "qabstracteventdispatcher.h"
 
 #ifndef QT_NO_DRAGANDDROP
 
@@ -71,6 +72,10 @@
 #include "qwidget_p.h"
 #include "qcursor_p.h"
 
+#ifndef QT_NO_XFIXES
+#include <X11/extensions/Xfixes.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 // #define DND_DEBUG
@@ -1112,21 +1117,6 @@ void qt_xdnd_send_leave()
     waiting_for_status = false;
 }
 
-// TODO: remove and use QApplication::currentKeyboardModifiers() in Qt 4.8.
-static Qt::KeyboardModifiers currentKeyboardModifiers()
-{
-    Window root;
-    Window child;
-    int root_x, root_y, win_x, win_y;
-    uint keybstate;
-    for (int i = 0; i < ScreenCount(X11->display); ++i) {
-        if (XQueryPointer(X11->display, QX11Info::appRootWindow(i), &root, &child,
-                          &root_x, &root_y, &win_x, &win_y, &keybstate))
-            return X11->translateModifiers(keybstate & 0x00ff);
-    }
-    return 0;
-}
-
 void QX11Data::xdndHandleDrop(QWidget *, const XEvent * xe, bool passive)
 {
     DEBUG("xdndHandleDrop");
@@ -1166,16 +1156,24 @@ void QX11Data::xdndHandleDrop(QWidget *, const XEvent * xe, bool passive)
         // some XEMBEDding, so try to find the real QMimeData used
         // based on the timestamp for this drop.
         QMimeData *dropData = 0;
-        int at = findXdndDropTransactionByTime(qt_xdnd_target_current_time);
-        if (at != -1)
+        const int at = findXdndDropTransactionByTime(qt_xdnd_target_current_time);
+        if (at != -1) {
             dropData = QDragManager::dragPrivate(X11->dndDropTransactions.at(at).object)->data;
+            // Can't use the source QMimeData if we need the image conversion code from xdndObtainData
+            if (dropData && dropData->hasImage())
+                dropData = 0;
+        }
         // if we can't find it, then use the data in the drag manager
-        if (!dropData)
-            dropData = (manager->object) ? manager->dragPrivate()->data : manager->dropData;
+        if (!dropData) {
+            if (manager->object && !manager->dragPrivate()->data->hasImage())
+                dropData = manager->dragPrivate()->data;
+            else
+                dropData = manager->dropData;
+        }
 
         // Drop coming from another app? Update keyboard modifiers.
         if (!qt_xdnd_dragging) {
-            QApplicationPrivate::modifier_buttons = currentKeyboardModifiers();
+            QApplicationPrivate::modifier_buttons = QApplication::queryKeyboardModifiers();
         }
 
         QDropEvent de(qt_xdnd_current_position, possible_actions, dropData,
@@ -1424,6 +1422,21 @@ void QDragManager::cancel(bool deleteSource)
     global_accepted_action = Qt::IgnoreAction;
 }
 
+static
+bool windowInteractsWithPosition(const QPoint & pos, Window w, int shapeType)
+{
+    int nrectanglesRet, dummyOrdering;
+    XRectangle *rectangles = XShapeGetRectangles(QX11Info::display(), w, shapeType, &nrectanglesRet, &dummyOrdering);
+    bool interacts = true;
+    if (rectangles) {
+        interacts = false;
+        for (int i = 0; !interacts && i < nrectanglesRet; ++i)
+            interacts = QRect(rectangles[i].x, rectangles[i].y, rectangles[i].width, rectangles[i].height).contains(pos);
+        XFree(rectangles);
+    }
+    return interacts;
+}
+
 static
 Window findRealWindow(const QPoint & pos, Window w, int md)
 {
@@ -1439,6 +1452,7 @@ Window findRealWindow(const QPoint & pos, Window w, int md)
 
         if (attr.map_state == IsViewable
             && QRect(attr.x,attr.y,attr.width,attr.height).contains(pos)) {
+            bool windowContainsMouse = true;
             {
                 Atom   type = XNone;
                 int f;
@@ -1448,8 +1462,13 @@ Window findRealWindow(const QPoint & pos, Window w, int md)
                 XGetWindowProperty(X11->display, w, ATOM(XdndAware), 0, 0, False,
                                    AnyPropertyType, &type, &f,&n,&a,&data);
                 if (data) XFree(data);
-                if (type)
-                    return w;
+                if (type) {
+                    // When ShapeInput and ShapeBounding are not set they return a single rectangle with the geometry of the window, this is why we
+                    // need an && here so that in the case one is set and the other is not we still get the correct result.
+                    windowContainsMouse = windowInteractsWithPosition(pos, w, ShapeInput) && windowInteractsWithPosition(pos, w, ShapeBounding);
+                    if (windowContainsMouse)
+                        return w;
+                }
             }
 
             Window r, p;
@@ -1470,7 +1489,10 @@ Window findRealWindow(const QPoint & pos, Window w, int md)
             }
 
             // No children!
-            return w;
+            if (!windowContainsMouse)
+                return 0;
+            else
+                return w;
         }
     }
     return 0;
@@ -1855,8 +1877,16 @@ static QVariant xdndObtainData(const char *format, QVariant::Type requestedType)
          && (!(w->windowType() == Qt::Desktop) || w->acceptDrops()))
     {
         QDragPrivate * o = QDragManager::self()->dragPrivate();
-        if (o->data->hasFormat(QLatin1String(format)))
-            result = o->data->data(QLatin1String(format));
+        const QString mimeType = QString::fromLatin1(format);
+        if (o->data->hasFormat(mimeType)) {
+            result = o->data->data(mimeType);
+        } else if (mimeType.startsWith(QLatin1String("image/")) && o->data->hasImage()) {
+            // ### duplicated from QInternalMimeData::renderDataHelper
+            QImage image = qvariant_cast<QImage>(o->data->imageData());
+            QBuffer buf(&result);
+            buf.open(QBuffer::WriteOnly);
+            image.save(&buf, mimeType.mid(mimeType.indexOf(QLatin1Char('/')) + 1).toLatin1().toUpper());
+        }
         return result;
     }
 
@@ -1940,7 +1970,10 @@ Qt::DropAction QDragManager::drag(QDrag * o)
         timer.start();
         do {
             XEvent event;
-            if (XCheckTypedEvent(X11->display, ClientMessage, &event))
+            // Pass the event through the event dispatcher filter so that applications
+            // which install an event filter on the dispatcher get to handle it first.
+            if (XCheckTypedEvent(X11->display, ClientMessage, &event) &&
+                !QAbstractEventDispatcher::instance()->filterEvent(&event))
                 qApp->x11ProcessEvent(&event);
 
             // sleep 50 ms, so we don't use up CPU cycles all the time.
