@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -243,52 +243,53 @@ bool QGestureManager::filterEventThroughContexts(const QMultiMap<QObject *,
     // TODO: sort contexts by the gesture type and check if one of the contexts
     //       is already active.
 
-    bool ret = false;
+    bool consumeEventHint = false;
 
     // filter the event through recognizers
     typedef QMultiMap<QObject *, Qt::GestureType>::const_iterator ContextIterator;
-    for (ContextIterator cit = contexts.begin(), ce = contexts.end(); cit != ce; ++cit) {
-        Qt::GestureType gestureType = cit.value();
+    ContextIterator contextEnd = contexts.end();
+    for (ContextIterator context = contexts.begin(); context != contextEnd; ++context) {
+        Qt::GestureType gestureType = context.value();
         QMap<Qt::GestureType, QGestureRecognizer *>::const_iterator
-                rit = m_recognizers.lowerBound(gestureType),
-                re = m_recognizers.upperBound(gestureType);
-        for (; rit != re; ++rit) {
-            QGestureRecognizer *recognizer = rit.value();
-            QObject *target = cit.key();
+                typeToRecognizerIterator = m_recognizers.lowerBound(gestureType),
+                typeToRecognizerEnd = m_recognizers.upperBound(gestureType);
+        for (; typeToRecognizerIterator != typeToRecognizerEnd; ++typeToRecognizerIterator) {
+            QGestureRecognizer *recognizer = typeToRecognizerIterator.value();
+            QObject *target = context.key();
             QGesture *state = getState(target, recognizer, gestureType);
             if (!state)
                 continue;
-            QGestureRecognizer::Result result = recognizer->recognize(state, target, event);
-            QGestureRecognizer::Result type = result & QGestureRecognizer::ResultState_Mask;
-            result &= QGestureRecognizer::ResultHint_Mask;
-            if (type == QGestureRecognizer::TriggerGesture) {
+            QGestureRecognizer::Result recognizerResult = recognizer->recognize(state, target, event);
+            QGestureRecognizer::Result recognizerState = recognizerResult & QGestureRecognizer::ResultState_Mask;
+            QGestureRecognizer::Result resultHint = recognizerResult & QGestureRecognizer::ResultHint_Mask;
+            if (recognizerState == QGestureRecognizer::TriggerGesture) {
                 DEBUG() << "QGestureManager:Recognizer: gesture triggered: " << state;
                 triggeredGestures << state;
-            } else if (type == QGestureRecognizer::FinishGesture) {
+            } else if (recognizerState == QGestureRecognizer::FinishGesture) {
                 DEBUG() << "QGestureManager:Recognizer: gesture finished: " << state;
                 finishedGestures << state;
-            } else if (type == QGestureRecognizer::MayBeGesture) {
+            } else if (recognizerState == QGestureRecognizer::MayBeGesture) {
                 DEBUG() << "QGestureManager:Recognizer: maybe gesture: " << state;
                 newMaybeGestures << state;
-            } else if (type == QGestureRecognizer::CancelGesture) {
+            } else if (recognizerState == QGestureRecognizer::CancelGesture) {
                 DEBUG() << "QGestureManager:Recognizer: not gesture: " << state;
                 notGestures << state;
-            } else if (type == QGestureRecognizer::Ignore) {
+            } else if (recognizerState == QGestureRecognizer::Ignore) {
                 DEBUG() << "QGestureManager:Recognizer: ignored the event: " << state;
             } else {
                 DEBUG() << "QGestureManager:Recognizer: hm, lets assume the recognizer"
                         << "ignored the event: " << state;
             }
-            if (result & QGestureRecognizer::ConsumeEventHint) {
+            if (resultHint & QGestureRecognizer::ConsumeEventHint) {
                 DEBUG() << "QGestureManager: we were asked to consume the event: "
                         << state;
-                ret = true;
+                consumeEventHint = true;
             }
         }
     }
     if (triggeredGestures.isEmpty() && finishedGestures.isEmpty()
         && newMaybeGestures.isEmpty() && notGestures.isEmpty())
-        return ret;
+        return consumeEventHint;
 
     QSet<QGesture *> startedGestures = triggeredGestures - m_activeGestures;
     triggeredGestures &= m_activeGestures;
@@ -390,7 +391,7 @@ bool QGestureManager::filterEventThroughContexts(const QMultiMap<QObject *,
     qDeleteAll(m_gesturesToDelete);
     m_gesturesToDelete.clear();
 
-    return ret;
+    return consumeEventHint;
 }
 
 // Cancel all gestures of children of the widget that original is associated with
@@ -565,7 +566,6 @@ void QGestureManager::getGestureTargets(const QSet<QGesture*> &gestures,
                         = w->d_func()->gestureContext.find(type);
                 if (it != w->d_func()->gestureContext.end()) {
                     // i.e. 'w' listens to gesture 'type'
-                    Qt::GestureFlags flags = it.value();
                     if (!(it.value() & Qt::DontStartGestureOnChildren) && w != widget) {
                         // conflicting gesture!
                         (*conflicts)[widget].append(gestures[widget]);
