@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -58,9 +58,7 @@
 #endif
 
 // This is necessary in order to be able to perform delayed invocation on slots
-// which take arguments of type WId.  One example is
-// QWidgetPrivate::_q_delayedDestroy, which is used to delay destruction of
-// CCoeControl objects until after the CONE event handler has finished running.
+// which take arguments of type WId.
 Q_DECLARE_METATYPE(WId)
 
 // Workaround for the fact that S60 SDKs 3.x do not contain the akntoolbar.h
@@ -84,20 +82,7 @@ QWidget *QWidgetPrivate::mouseGrabber = 0;
 QWidget *QWidgetPrivate::keyboardGrabber = 0;
 CEikButtonGroupContainer *QS60Data::cba = 0;
 
-static bool isEqual(const QList<QAction*>& a, const QList<QAction*>& b)
-{
-    if ( a.count() != b.count())
-        return false;
-    int index=0;
-    while (index<a.count()) {
-        if (a.at(index)->softKeyRole() != b.at(index)->softKeyRole())
-            return false;
-        if (a.at(index)->text().compare(b.at(index)->text())!=0)
-            return false;
-        index++;
-    }
-    return true;
-}
+int qt_symbian_create_desktop_on_screen = -1;
 
 void QWidgetPrivate::setWSGeometry(bool dontShow, const QRect &)
 {
@@ -231,13 +216,23 @@ void QWidgetPrivate::setGeometry_sys(int x, int y, int w, int h, bool isMove)
 
     QPoint oldPos(q->pos());
     QSize oldSize(q->size());
-    QRect oldGeom(data.crect);
+
+    bool checkExtra = true;
+    if (q->isWindow() && (data.window_state & (Qt::WindowFullScreen | Qt::WindowMaximized))) {
+        // Do not allow fullscreen/maximized windows to expand beyond client rect
+        TRect r = S60->clientRect();
+        w = qMin(w, r.Width());
+        h = qMin(h, r.Height());
+
+        if (w == r.Width() && h == r.Height())
+            checkExtra = false;
+    }
 
     // Lose maximized status if deliberate resize
     if (w != oldSize.width() || h != oldSize.height())
         data.window_state &= ~Qt::WindowMaximized;
 
-    if (extra) {                                // any size restrictions?
+    if (checkExtra && extra) {  // any size restrictions?
         w = qMin(w,extra->maxw);
         h = qMin(h,extra->maxh);
         w = qMax(w,extra->minw);
@@ -276,6 +271,8 @@ void QWidgetPrivate::setGeometry_sys(int x, int y, int w, int h, bool isMove)
             q->internalWinId()->SetRect(TRect(TPoint(x, y), TSize(w, h)));
             topData()->normalGeometry = data.crect;
         }
+        QSymbianControl *window = static_cast<QSymbianControl *>(q->internalWinId());
+        window->ensureFixNativeOrientation();
     } else {
         data.crect.setRect(x, y, w, h);
 
@@ -335,26 +332,28 @@ void QWidgetPrivate::create_sys(WId window, bool /* initializeWindow */, bool de
 
     bool topLevel = (flags & Qt::Window);
     bool popup = (type == Qt::Popup);
-    bool dialog = (type == Qt::Dialog
-                   || type == Qt::Sheet
-                   || (flags & Qt::MSWindowsFixedSizeDialogHint));
     bool desktop = (type == Qt::Desktop);
-    //bool tool = (type == Qt::Tool || type == Qt::Drawer);
 
     if (popup)
         flags |= Qt::WindowStaysOnTopHint; // a popup stays on top
 
-    TRect clientRect = static_cast<CEikAppUi*>(S60->appUi())->ClientRect();
+    TRect clientRect = S60->clientRect();
     int sw = clientRect.Width();
     int sh = clientRect.Height();
 
     if (desktop) {
-        TSize screenSize = S60->screenDevice()->SizeInPixels();
+        symbianScreenNumber = qMax(qt_symbian_create_desktop_on_screen, 0);
+        TSize screenSize = S60->screenDevice(symbianScreenNumber)->SizeInPixels();
         data.crect.setRect(0, 0, screenSize.iWidth, screenSize.iHeight);
         q->setAttribute(Qt::WA_DontShowOnScreen);
     } else if (topLevel && !q->testAttribute(Qt::WA_Resized)){
         int width = sw;
         int height = sh;
+        if (symbianScreenNumber > 0) {
+            TSize screenSize = S60->screenDevice(symbianScreenNumber)->SizeInPixels();
+            width = screenSize.iWidth;
+            height = screenSize.iHeight;
+        }
         if (extra) {
             width = qMax(qMin(width, extra->maxw), extra->minw);
             height = qMax(qMin(height, extra->maxh), extra->minh);
@@ -374,7 +373,9 @@ void QWidgetPrivate::create_sys(WId window, bool /* initializeWindow */, bool de
         if (!q->testAttribute(Qt::WA_Moved) && !q->testAttribute(Qt::WA_DontShowOnScreen))
             data.crect.moveTopLeft(QPoint(clientRect.iTl.iX, clientRect.iTl.iY));
 
-        QScopedPointer<QSymbianControl> control( q_check_ptr(new QSymbianControl(q)) );
+        QScopedPointer<QSymbianControl> control( new QSymbianControl(q) );
+        Q_CHECK_PTR(control);
+
         QT_TRAP_THROWING(control->ConstructL(true, desktop));
         control->SetMopParent(static_cast<CEikAppUi*>(S60->appUi()));
 
@@ -392,16 +393,18 @@ void QWidgetPrivate::create_sys(WId window, bool /* initializeWindow */, bool de
                 stackingFlags = ECoeStackFlagStandard;
             }
             control->MakeVisible(false);
-            QT_TRAP_THROWING(control->ControlEnv()->AppUi()->AddToStackL(control.data(), ECoeStackPriorityDefault, stackingFlags));
-            // Avoid keyboard focus to a hidden window.
-            control->setFocusSafely(false);
 
-            RDrawableWindow *const drawableWindow = control->DrawableWindow();
-            // Request mouse move events.
-            drawableWindow->PointerFilter(EPointerFilterEnterExit
-                | EPointerFilterMove | EPointerFilterDrag, 0);
-            drawableWindow->EnableVisibilityChangeEvents();
+            if (!isGLGlobalShareWidget) {
+                QT_TRAP_THROWING(control->ControlEnv()->AppUi()->AddToStackL(control.data(), ECoeStackPriorityDefault, stackingFlags));
+                // Avoid keyboard focus to a hidden window.
+                control->setFocusSafely(false);
 
+                RDrawableWindow *const drawableWindow = control->DrawableWindow();
+                // Request mouse move events.
+                drawableWindow->PointerFilter(EPointerFilterEnterExit
+                    | EPointerFilterMove | EPointerFilterDrag, 0);
+                drawableWindow->EnableVisibilityChangeEvents();
+            }
         }
 
         q->setAttribute(Qt::WA_WState_Created);
@@ -419,7 +422,9 @@ void QWidgetPrivate::create_sys(WId window, bool /* initializeWindow */, bool de
 
     } else if (q->testAttribute(Qt::WA_NativeWindow) || paintOnScreen()) { // create native child widget
 
-        QScopedPointer<QSymbianControl> control( q_check_ptr(new QSymbianControl(q)) );
+        QScopedPointer<QSymbianControl> control( new QSymbianControl(q) );
+        Q_CHECK_PTR(control);
+
         QT_TRAP_THROWING(control->ConstructL(!parentWidget));
 
         // Symbian windows are always created in an inactive state
@@ -466,8 +471,8 @@ void QWidgetPrivate::create_sys(WId window, bool /* initializeWindow */, bool de
         // Delay deletion of the control in case this function is called in the
         // context of a CONE event handler such as
         // CCoeControl::ProcessPointerEventL
-        QMetaObject::invokeMethod(q, "_q_delayedDestroy",
-            Qt::QueuedConnection, Q_ARG(WId, destroyw));
+        widCleanupList << destroyw;
+        QMetaObject::invokeMethod(q, "_q_cleanupWinIds", Qt::QueuedConnection);
     }
 
     if (q->testAttribute(Qt::WA_AcceptTouchEvents))
@@ -502,71 +507,54 @@ void QWidgetPrivate::show_sys()
 #ifdef Q_WS_S60
         // Lazily initialize the S60 screen furniture when the first window is shown.
         if (q->isWindow() && !QApplication::testAttribute(Qt::AA_S60DontConstructApplicationPanes)
-                && !S60->buttonGroupContainer() && !S60->statusPane()) {
+            && !q->testAttribute(Qt::WA_DontShowOnScreen) && !S60->screenFurnitureFullyCreated) {
+            // Create the status pane and CBA here if not yet done. These could be created earlier
+            // if application was launched in "App-Lite" version
+            if (!S60->buttonGroupContainer() && !S60->statusPane())
+                S60->createStatusPaneAndCBA();
 
-            if (!q->testAttribute(Qt::WA_DontShowOnScreen)) {
+            if (S60->buttonGroupContainer()) {
+                if (isFullscreen && !cbaRequested)
+                    S60->buttonGroupContainer()->MakeVisible(false);
+            }
 
-                // Create the status pane and CBA here
-                CEikAppUi *ui = static_cast<CEikAppUi *>(S60->appUi());
-                MEikAppUiFactory *factory = CEikonEnv::Static()->AppUiFactory();
+            // If the creation of the first widget is delayed, for example by doing it
+            // inside the event loop, S60 somehow "forgets" to set the visibility of the
+            // toolbar (the three middle softkeys) when you flip the phone over, so we
+            // need to do it ourselves to avoid a "hole" in the application, even though
+            // Qt itself does not use the toolbar directly..
+            CAknAppUi *appui = dynamic_cast<CAknAppUi *>(CEikonEnv::Static()->AppUi());
+            if (appui) {
+                CAknToolbar *toolbar = appui->PopupToolbar();
+                if (toolbar && !toolbar->IsVisible())
+                    toolbar->SetToolbarVisibility(ETrue);
+            }
 
-                QT_TRAP_THROWING(
-                    factory->CreateResourceIndependentFurnitureL(ui);
-
-                    TRect boundingRect = static_cast<CEikAppUi*>(S60->appUi())->ClientRect();
-
-                    CEikButtonGroupContainer *cba = CEikButtonGroupContainer::NewL(CEikButtonGroupContainer::ECba,
-                        CEikButtonGroupContainer::EHorizontal,ui,R_AVKON_SOFTKEYS_EMPTY_WITH_IDS);
-                    if (isFullscreen && !cbaRequested)
-                        cba->MakeVisible(false);
-
-                    CEikButtonGroupContainer *oldCba = factory->SwapButtonGroup(cba);
-                    Q_ASSERT(!oldCba);
-                    S60->setButtonGroupContainer(cba);
-
-                    // If the creation of the first widget is delayed, for example by doing it
-                    // inside the event loop, S60 somehow "forgets" to set the visibility of the
-                    // toolbar (the three middle softkeys) when you flip the phone over, so we
-                    // need to do it ourselves to avoid a "hole" in the application, even though
-                    // Qt itself does not use the toolbar directly..
-                    CAknAppUi *appui = dynamic_cast<CAknAppUi *>(CEikonEnv::Static()->AppUi());
-                    if (appui) {
-                        CAknToolbar *toolbar = appui->PopupToolbar();
-                        if (toolbar && !toolbar->IsVisible())
-                            toolbar->SetToolbarVisibility(ETrue);
-                    }
-
-                    CEikMenuBar *menuBar = new(ELeave) CEikMenuBar;
-                    menuBar->ConstructL(ui, 0, R_AVKON_MENUPANE_EMPTY);
-                    menuBar->SetMenuType(CEikMenuBar::EMenuOptions);
-                    S60->appUi()->AddToStackL(menuBar,ECoeStackPriorityMenu,ECoeStackFlagRefusesFocus);
-
-                    CEikMenuBar *oldMenu = factory->SwapMenuBar(menuBar);
-                    Q_ASSERT(!oldMenu);
-                )
-
-                if (S60->statusPane()) {
-                    // Use QDesktopWidget as the status pane observer to proxy for the AppUi.
-                    // Can't use AppUi directly because it privately inherits from MEikStatusPaneObserver.
-                    QSymbianControl *desktopControl = static_cast<QSymbianControl *>(QApplication::desktop()->winId());
-                    S60->statusPane()->SetObserver(desktopControl);
-                    if (isFullscreen) {
-                        const bool cbaVisible = S60->buttonGroupContainer() && S60->buttonGroupContainer()->IsVisible();
-                        S60->setStatusPaneAndButtonGroupVisibility(false, cbaVisible);
+            if (S60->statusPane()) {
+               if (isFullscreen) {
+                    const bool cbaVisible = S60->buttonGroupContainer() && S60->buttonGroupContainer()->IsVisible();
+                    S60->setStatusPaneAndButtonGroupVisibility(false, cbaVisible);
+                    if (cbaVisible) {
+                        // Fix window dimensions as without screen furniture they will have
+                        // defaulted to full screen dimensions initially.
+                        id->handleClientAreaChange();
                     }
                 }
             }
+            S60->screenFurnitureFullyCreated = true;
         }
 #endif
 
         // Fill client area if maximized OR
         // Put window below status pane unless the window has an explicit position.
         if (!isFullscreen) {
+            // Use QS60Data::clientRect to take into account that native keyboard
+            // might affect ClientRect() return value.
             if (q->windowState() & Qt::WindowMaximized) {
-                TRect r = static_cast<CEikAppUi*>(S60->appUi())->ClientRect();
+                TRect r = S60->clientRect();
                 id->SetExtent(r.iTl, r.Size());
             } else if (!q->testAttribute(Qt::WA_Moved) && q->windowType() != Qt::Dialog) {
-                id->SetPosition(static_cast<CEikAppUi*>(S60->appUi())->ClientRect().iTl);
+                id->SetPosition(S60->clientRect().iTl);
             }
         }
 
@@ -640,7 +628,7 @@ void QWidgetPrivate::raise_sys()
 
         // If toplevel widget, raise app to foreground
         if (q->isWindow())
-            S60->wsSession().SetWindowGroupOrdinalPosition(S60->windowGroup().Identifier(), 0);
+            S60->wsSession().SetWindowGroupOrdinalPosition(S60->windowGroup(q).Identifier(), 0);
     }
 }
 
@@ -652,7 +640,7 @@ void QWidgetPrivate::lower_sys()
     if (q->internalWinId()) {
         // If toplevel widget, lower app to background
         if (q->isWindow())
-            S60->wsSession().SetWindowGroupOrdinalPosition(S60->windowGroup().Identifier(), -1);
+            S60->wsSession().SetWindowGroupOrdinalPosition(S60->windowGroup(q).Identifier(), -1);
         else
             q->internalWinId()->DrawableWindow()->SetOrdinalPosition(-1);
     }
@@ -722,6 +710,11 @@ void QWidgetPrivate::setParent_sys(QWidget *parent, Qt::WindowFlags f)
 {
     Q_Q(QWidget);
 
+    if (parent && parent->windowType() == Qt::Desktop) {
+        symbianScreenNumber = qt_widget_private(parent)->symbianScreenNumber;
+        parent = 0;
+    }
+
     bool wasCreated = q->testAttribute(Qt::WA_WState_Created);
 
     if (q->isVisible() && q->parentWidget() && parent != q->parentWidget())
@@ -762,7 +755,7 @@ void QWidgetPrivate::setParent_sys(QWidget *parent, Qt::WindowFlags f)
     adjustFlags(data.window_flags, q);
     // keep compatibility with previous versions, we need to preserve the created state
     // (but we recreate the winId for the widget being reparented, again for compatibility)
-    if (wasCreated || (!q->isWindow() && parent->testAttribute(Qt::WA_WState_Created)))
+    if (wasCreated || (!q->isWindow() && parent && parent->testAttribute(Qt::WA_WState_Created)))
         createWinId();
     if (q->isWindow() || (!parent || parent->isVisible()) || explicitlyHidden)
         q->setAttribute(Qt::WA_WState_Hidden);
@@ -791,37 +784,62 @@ void QWidgetPrivate::setConstraints_sys()
 void QWidgetPrivate::s60UpdateIsOpaque()
 {
     Q_Q(QWidget);
-
-    if (!q->testAttribute(Qt::WA_WState_Created) || !q->testAttribute(Qt::WA_TranslucentBackground))
+    if (!q->testAttribute(Qt::WA_WState_Created))
         return;
-
+    const bool writeAlpha = extraData()->nativePaintMode == QWExtra::BlitWriteAlpha;
+    const bool requireAlphaChannel = !isOpaque || writeAlpha;
     createTLExtra();
-
     RWindow *const window = static_cast<RWindow *>(q->effectiveWinId()->DrawableWindow());
-
 #ifdef Q_SYMBIAN_SEMITRANSPARENT_BG_SURFACE
-    window->SetSurfaceTransparency(!isOpaque);
-    extra->topextra->nativeWindowTransparencyEnabled = !isOpaque;
-#else
-    if (!isOpaque) {
-        const TDisplayMode displayMode = static_cast<TDisplayMode>(window->SetRequiredDisplayMode(EColor16MA));
-        if (window->SetTransparencyAlphaChannel() == KErrNone) {
-            window->SetBackgroundColor(TRgb(255, 255, 255, 0));
-            extra->topextra->nativeWindowTransparencyEnabled = 1;
-
-            if (extra->topextra->backingStore.data() &&
-                    QApplicationPrivate::graphics_system_name == QLatin1String("openvg")) {
-                // Semi-transparent EGL surfaces aren't supported. We need to
-                // recreate backing store to get translucent surface (raster surface).
-                extra->topextra->backingStore.create(q);
-                extra->topextra->backingStore.registerWidget(q);
-            }
-        }
-    } else if (extra->topextra->nativeWindowTransparencyEnabled) {
-        window->SetTransparentRegion(TRegionFix<1>());
-        extra->topextra->nativeWindowTransparencyEnabled = 0;
+    if (QApplicationPrivate::instance()->useTranslucentEGLSurfaces
+            && !extra->topextra->forcedToRaster) {
+        window->SetSurfaceTransparency(!isOpaque);
+        extra->topextra->nativeWindowTransparencyEnabled = !isOpaque;
+        return;
     }
 #endif
+    const bool recreateBackingStore = extra->topextra->backingStore.data() && (
+                                          QApplicationPrivate::graphics_system_name == QLatin1String("openvg") ||
+                                          QApplicationPrivate::graphics_system_name == QLatin1String("opengl")
+                                      );
+    if (requireAlphaChannel) {
+        window->SetRequiredDisplayMode(EColor16MA);
+        if (window->SetTransparencyAlphaChannel() == KErrNone)
+            window->SetBackgroundColor(TRgb(255, 255, 255, 0));
+    } else {
+        if (recreateBackingStore) {
+            // Clear the UI surface to ensure that the EGL surface content is visible
+            CWsScreenDevice *screenDevice = S60->screenDevice(q);
+            QScopedPointer<CWindowGc> gc(new CWindowGc(screenDevice));
+            const int err = gc->Construct();
+            if (!err) {
+                gc->Activate(*window);
+                window->BeginRedraw();
+                gc->SetDrawMode(CWindowGc::EDrawModeWriteAlpha);
+                gc->SetBrushColor(TRgb(0, 0, 0, 0));
+                gc->Clear(TRect(0, 0, q->width(), q->height()));
+                window->EndRedraw();
+            }
+        }
+        if (extra->topextra->nativeWindowTransparencyEnabled)
+            window->SetTransparentRegion(TRegionFix<1>());
+    }
+    extra->topextra->nativeWindowTransparencyEnabled = requireAlphaChannel;
+    if (recreateBackingStore) {
+        extra->topextra->backingStore.create(q);
+        extra->topextra->backingStore.registerWidget(q);
+        bool noSystemRotationDisabled = false;
+        if (requireAlphaChannel) {
+            if (q->testAttribute(Qt::WA_SymbianNoSystemRotation)) {
+                // FixNativeOrientation() will not work without an EGL surface
+                q->setAttribute(Qt::WA_SymbianNoSystemRotation, false);
+                noSystemRotationDisabled = true;
+            }
+        } else {
+            q->setAttribute(Qt::WA_SymbianNoSystemRotation, extra->topextra->noSystemRotationDisabled);
+        }
+        extra->topextra->noSystemRotationDisabled = noSystemRotationDisabled;
+    }
 }
 
 void QWidgetPrivate::setWindowIcon_sys(bool forceReset)
@@ -980,6 +998,8 @@ void QWidgetPrivate::createTLSysExtra()
 {
     extra->topextra->inExpose = 0;
     extra->topextra->nativeWindowTransparencyEnabled = 0;
+    extra->topextra->forcedToRaster = 0;
+    extra->topextra->noSystemRotationDisabled = 0;
 }
 
 void QWidgetPrivate::deleteTLSysExtra()
@@ -1022,8 +1042,12 @@ void QWidgetPrivate::registerTouchWindow()
         RWindow *rwindow = static_cast<RWindow *>(q->effectiveWinId()->DrawableWindow());
         QSymbianControl *window = static_cast<QSymbianControl *>(q->effectiveWinId());
         //Enabling advanced pointer events for controls that already have active windows causes a panic.
-        if (!window->isControlActive())
+        if (!window->isControlActive()) {
             rwindow->EnableAdvancedPointers();
+#ifdef COE_GROUPED_POINTER_EVENT_VERSION
+            qt_symbian_throwIfError(window->ConfigureEventData(CCoeControl::EEventDataAllowGroupedPointerEvents));
+#endif
+        }
     }
 #endif
 }
@@ -1031,13 +1055,13 @@ void QWidgetPrivate::registerTouchWindow()
 int QWidget::metric(PaintDeviceMetric m) const
 {
     Q_D(const QWidget);
-    int val;
+    int val = 0;
     if (m == PdmWidth) {
         val = data->crect.width();
     } else if (m == PdmHeight) {
         val = data->crect.height();
     } else {
-        CWsScreenDevice *scr = S60->screenDevice();
+        CWsScreenDevice *scr = S60->screenDevice(this);
         switch(m) {
         case PdmDpiX:
         case PdmPhysicalDpiX:
@@ -1184,17 +1208,10 @@ void QWidget::setWindowState(Qt::WindowStates newstate)
         }
 
 #ifdef Q_WS_S60
-        bool decorationsVisible(false);
-        if (!parentWidget()) { // Only top level native windows have control over cba/status pane
-            // Hide window decoration when switching to fullscreen / minimized otherwise show decoration.
-            // The window decoration visibility has to be changed before doing actual window state
-            // change since in that order the availableGeometry will return directly the right size and
-            // we will avoid unnecessary redraws
-            decorationsVisible = !(newstate & (Qt::WindowFullScreen | Qt::WindowMinimized));
-            const bool statusPaneVisibility = decorationsVisible;
-            const bool buttonGroupVisibility = (decorationsVisible || (isFullscreen && cbaRequested));
-            S60->setStatusPaneAndButtonGroupVisibility(statusPaneVisibility, buttonGroupVisibility);
-        }
+        // The window decoration visibility has to be changed before doing actual window state
+        // change since in that order the availableGeometry will return directly the right size and
+        // we will avoid unnecessary redraws
+        bool decorationsVisible = S60->setRecursiveDecorationsVisibility(this, newstate);
 #endif // Q_WS_S60
 
         // Ensure the initial size is valid, since we store it as normalGeometry below.
@@ -1207,7 +1224,16 @@ void QWidget::setWindowState(Qt::WindowStates newstate)
         const bool cbaVisibilityHint = windowFlags() & Qt::WindowSoftkeysVisibleHint;
         if (newstate & Qt::WindowFullScreen && !cbaVisibilityHint) {
             setAttribute(Qt::WA_OutsideWSRange, false);
-            window->SetExtentToWholeScreen();
+            if (d->symbianScreenNumber > 0) {
+                int w = S60->screenWidthInPixelsForScreen[d->symbianScreenNumber];
+                int h = S60->screenHeightInPixelsForScreen[d->symbianScreenNumber];
+                if (w <= 0 || h <= 0)
+                    window->SetExtentToWholeScreen();
+                else
+                    window->SetExtent(TPoint(0, 0), TSize(w, h));
+            } else {
+                window->SetExtentToWholeScreen();
+            }
         } else if (newstate & Qt::WindowMaximized || ((newstate & Qt::WindowFullScreen) && cbaVisibilityHint)) {
             setAttribute(Qt::WA_OutsideWSRange, false);
             TRect maxExtent = qt_QRect2TRect(qApp->desktop()->availableGeometry(this));
@@ -1219,7 +1245,7 @@ void QWidget::setWindowState(Qt::WindowStates newstate)
             // normal mode after showing the status pane, the geometry would overlap so we should
             // move it if it never had an explicit position.
             if (!wasMoved && S60->statusPane() && decorationsVisible) {
-                TPoint tl = static_cast<CEikAppUi*>(S60->appUi())->ClientRect().iTl;
+                TPoint tl = S60->clientRect().iTl;
                 normalGeometry.setTopLeft(QPoint(tl.iX, tl.iY));
             }
 #endif
@@ -1244,6 +1270,12 @@ void QWidget::setWindowState(Qt::WindowStates newstate)
     if (newstate & Qt::WindowActive)
         activateWindow();
 
+    if (isWindow()) {
+        // Now that the new state is set, fix the display memory layout, if needed.
+        QSymbianControl *window = static_cast<QSymbianControl *>(effectiveWinId());
+        window->ensureFixNativeOrientation();
+    }
+
     QWindowStateChangeEvent e(oldstate);
     QApplication::sendEvent(this, &e);
 }
@@ -1390,7 +1422,8 @@ void QWidget::activateWindow()
     if (tlw->isVisible()) {
         window()->createWinId();
         QSymbianControl *id = static_cast<QSymbianControl *>(tlw->internalWinId());
-        id->setFocusSafely(true);
+        if (!id->IsFocused())
+            id->setFocusSafely(true);
     }
 }
 
