@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -60,6 +60,7 @@
 #include <qdir.h>
 #include <qapplication.h>
 #include "qtextcontrol_p.h"
+#include "qfont_p.h"
 #include "private/qtextedit_p.h"
 #include "private/qdataurl_p.h"
 
@@ -158,7 +159,7 @@ bool Qt::mightBeRichText(const QString& text)
 QString Qt::escape(const QString& plain)
 {
     QString rich;
-    rich.reserve(int(plain.length() * 1.1));
+    rich.reserve(int(plain.length() * qreal(1.1)));
     for (int i = 0; i < plain.length(); ++i) {
         if (plain.at(i) == QLatin1Char('<'))
             rich += QLatin1String("&lt;");
@@ -584,6 +585,29 @@ void QTextDocument::setDefaultTextOption(const QTextOption &option)
         d->lout->documentChanged(0, 0, d->length());
 }
 
+/*!
+    \since 4.8
+
+    The default cursor movement style is used by all QTextCursor objects
+    created from the document. The default is Qt::LogicalMoveStyle.
+*/
+Qt::CursorMoveStyle QTextDocument::defaultCursorMoveStyle() const
+{
+    Q_D(const QTextDocument);
+    return d->defaultCursorMoveStyle;
+}
+
+/*!
+    \since 4.8
+
+    Sets the default cursor movement style to the given \a style.
+*/
+void QTextDocument::setDefaultCursorMoveStyle(Qt::CursorMoveStyle style)
+{
+    Q_D(QTextDocument);
+    d->defaultCursorMoveStyle = style;
+}
+
 /*!
     \fn void QTextDocument::markContentsDirty(int position, int length)
 
@@ -1671,8 +1695,6 @@ static void printPage(int index, QPainter *painter, const QTextDocument *doc, co
     painter->restore();
 }
 
-Q_GUI_EXPORT extern int qt_defaultDpi();
-
 /*!
     Prints the document to the given \a printer. The QPrinter must be
     set up before being used with this function.
@@ -1985,6 +2007,8 @@ QVariant QTextDocument::loadResource(int type, const QUrl &name)
             if (fi.exists()) {
                 resourceUrl =
                     QUrl::fromLocalFile(fi.absolutePath() + QDir::separator()).resolved(name);
+            } else if (currentURL.isEmpty()) {
+                resourceUrl.setScheme(QLatin1String("file"));
             }
         }
 
@@ -2075,6 +2099,10 @@ QString QTextHtmlExporter::toHtml(const QByteArray &encoding, ExportMode mode)
             html += QLatin1String(" font-size:");
             html += QString::number(defaultCharFormat.fontPointSize());
             html += QLatin1String("pt;");
+        } else if (defaultCharFormat.hasProperty(QTextFormat::FontPixelSize)) {
+            html += QLatin1String(" font-size:");
+            html += QString::number(defaultCharFormat.intProperty(QTextFormat::FontPixelSize));
+            html += QLatin1String("px;");
         }
 
         html += QLatin1String(" font-weight:");
@@ -2155,6 +2183,10 @@ bool QTextHtmlExporter::emitCharFormatStyle(const QTextCharFormat &format)
             html += QLatin1Char(';');
             attributesEmitted = true;
         }
+    } else if (format.hasProperty(QTextFormat::FontPixelSize)) {
+        html += QLatin1String(" font-size:");
+        html += QString::number(format.intProperty(QTextFormat::FontPixelSize));
+        html += QLatin1String("px;");
     }
 
     if (format.hasProperty(QTextFormat::FontWeight)
@@ -2602,14 +2634,40 @@ void QTextHtmlExporter::emitBlock(const QTextBlock &block)
                 default: html += QLatin1String("<ul"); // ### should not happen
             }
 
-            html += QLatin1String(" style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px;");
+            QString styleString = QString::fromLatin1("margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px;");
 
             if (format.hasProperty(QTextFormat::ListIndent)) {
-                html += QLatin1String(" -qt-list-indent: ");
-                html += QString::number(format.indent());
-                html += QLatin1Char(';');
+                styleString += QLatin1String(" -qt-list-indent: ");
+                styleString += QString::number(format.indent());
+                styleString += QLatin1Char(';');
             }
 
+            if (format.hasProperty(QTextFormat::ListNumberPrefix)) {
+                QString numberPrefix = format.numberPrefix();
+                numberPrefix.replace(QLatin1Char('"'), QLatin1String("\\22"));
+                numberPrefix.replace(QLatin1Char('\''), QLatin1String("\\27")); // FIXME: There's a problem in the CSS parser the prevents this from being correctly restored
+                styleString += QLatin1String(" -qt-list-number-prefix: ");
+                styleString += QLatin1Char('\'');
+                styleString += numberPrefix;
+                styleString += QLatin1Char('\'');
+                styleString += QLatin1Char(';');
+            }
+
+            if (format.hasProperty(QTextFormat::ListNumberSuffix)) {
+                if (format.numberSuffix() != QLatin1String(".")) { // this is our default
+                    QString numberSuffix = format.numberSuffix();
+                    numberSuffix.replace(QLatin1Char('"'), QLatin1String("\\22"));
+                    numberSuffix.replace(QLatin1Char('\''), QLatin1String("\\27")); // see above
+                    styleString += QLatin1String(" -qt-list-number-suffix: ");
+                    styleString += QLatin1Char('\'');
+                    styleString += numberSuffix;
+                    styleString += QLatin1Char('\'');
+                    styleString += QLatin1Char(';');
+                }
+            }
+
+            html += QLatin1String(" style=\"");
+            html += styleString;
             html += QLatin1String("\">");
         }
 
@@ -2651,6 +2709,8 @@ void QTextHtmlExporter::emitBlock(const QTextBlock &block)
     emitBlockAttributes(block);
 
     html += QLatin1Char('>');
+    if (block.begin().atEnd())
+        html += QLatin1String("<br />");
 
     QTextBlock::Iterator it = block.begin();
     if (fragmentMarkers && !it.atEnd() && block == doc->begin())
