@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -129,9 +129,9 @@
     \value NoProxy No proxying is used
     \value DefaultProxy Proxy is determined based on the application proxy set using setApplicationProxy()
     \value Socks5Proxy \l Socks5 proxying is used
-    \value HttpProxy HTTP transparent proxying is used
-    \value HttpCachingProxy Proxying for HTTP requests only
-    \value FtpCachingProxy Proxying for FTP requests only
+    \value HttpProxy HTTP transparent proxying is used (This value was introduced in 4.3.)
+    \value HttpCachingProxy Proxying for HTTP requests only (This value was introduced in 4.4.)
+    \value FtpCachingProxy Proxying for FTP requests only (This value was introduced in 4.4.)
 
     The table below lists different proxy types and their
     capabilities. Since each proxy type has different capabilities, it
@@ -228,6 +228,10 @@
 #include "qmutex.h"
 #include "qurl.h"
 
+#ifndef QT_NO_BEARERMANAGEMENT
+#include <QtNetwork/QNetworkConfiguration>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 class QSocks5SocketEngineHandler;
@@ -243,6 +247,12 @@ public:
         , socks5SocketEngineHandler(0)
         , httpSocketEngineHandler(0)
     {
+#ifndef QT_NO_SOCKS5
+        socks5SocketEngineHandler = new QSocks5SocketEngineHandler();
+#endif
+#ifndef QT_NO_HTTP
+        httpSocketEngineHandler = new QHttpSocketEngineHandler();
+#endif
     }
 
     ~QGlobalNetworkProxy()
@@ -253,19 +263,6 @@ public:
         delete httpSocketEngineHandler;
     }
 
-    void init()
-    {
-        QMutexLocker lock(&mutex);
-#ifndef QT_NO_SOCKS5
-        if (!socks5SocketEngineHandler)
-            socks5SocketEngineHandler = new QSocks5SocketEngineHandler();
-#endif
-#ifndef QT_NO_HTTP
-        if (!httpSocketEngineHandler)
-            httpSocketEngineHandler = new QHttpSocketEngineHandler();
-#endif
-    }
-
     void setApplicationProxy(const QNetworkProxy &proxy)
     {
         QMutexLocker lock(&mutex);
@@ -426,8 +423,6 @@ template<> void QSharedDataPointer<QNetworkProxyPrivate>::detach()
 QNetworkProxy::QNetworkProxy()
     : d(0)
 {
-    if (QGlobalNetworkProxy *globalProxy = globalNetworkProxy())
-        globalProxy->init();
 }
 
 /*!
@@ -442,8 +437,6 @@ QNetworkProxy::QNetworkProxy(ProxyType type, const QString &hostName, quint16 po
                   const QString &user, const QString &password)
     : d(new QNetworkProxyPrivate(type, hostName, port, user, password))
 {
-    if (QGlobalNetworkProxy *globalProxy = globalNetworkProxy())
-        globalProxy->init();
 }
 
 /*!
@@ -716,6 +709,9 @@ public:
     QUrl remote;
     int localPort;
     QNetworkProxyQuery::QueryType type;
+#ifndef QT_NO_BEARERMANAGEMENT
+    QNetworkConfiguration config;
+#endif
 };
 
 template<> void QSharedDataPointer<QNetworkProxyQueryPrivate>::detach()
@@ -777,6 +773,11 @@ template<> void QSharedDataPointer<QNetworkProxyQueryPrivate>::detach()
     like choosing an caching HTTP proxy for HTTP-based connections,
     but a more powerful SOCKSv5 proxy for all others.
 
+    The network configuration specifies which configuration to use,
+    when bearer management is used. For example on a mobile phone
+    the proxy settings are likely to be different for the cellular
+    network vs WLAN.
+
     Some of the criteria may not make sense in all of the types of
     query. The following table lists the criteria that are most
     commonly used, according to the type of query.
@@ -902,6 +903,74 @@ QNetworkProxyQuery::QNetworkProxyQuery(quint16 bindPort, const QString &protocol
     d->type = queryType;
 }
 
+#ifndef QT_NO_BEARERMANAGEMENT
+/*!
+    \since 4.8
+
+    Constructs a QNetworkProxyQuery with the URL \a requestUrl and
+    sets the query type to \a queryType. The specified \a networkConfiguration
+    is used to resolve the proxy settings.
+
+    \sa protocolTag(), peerHostName(), peerPort(), networkConfiguration()
+*/
+QNetworkProxyQuery::QNetworkProxyQuery(const QNetworkConfiguration &networkConfiguration,
+                                       const QUrl &requestUrl, QueryType queryType)
+{
+    d->config = networkConfiguration;
+    d->remote = requestUrl;
+    d->type = queryType;
+}
+
+/*!
+    \since 4.8
+
+    Constructs a QNetworkProxyQuery of type \a queryType and sets the
+    protocol tag to be \a protocolTag. This constructor is suitable
+    for QNetworkProxyQuery::TcpSocket queries, because it sets the
+    peer hostname to \a hostname and the peer's port number to \a
+    port. The specified \a networkConfiguration
+    is used to resolve the proxy settings.
+
+    \sa networkConfiguration()
+*/
+QNetworkProxyQuery::QNetworkProxyQuery(const QNetworkConfiguration &networkConfiguration,
+                                       const QString &hostname, int port,
+                                       const QString &protocolTag,
+                                       QueryType queryType)
+{
+    d->config = networkConfiguration;
+    d->remote.setScheme(protocolTag);
+    d->remote.setHost(hostname);
+    d->remote.setPort(port);
+    d->type = queryType;
+}
+
+/*!
+    \since 4.8
+
+    Constructs a QNetworkProxyQuery of type \a queryType and sets the
+    protocol tag to be \a protocolTag. This constructor is suitable
+    for QNetworkProxyQuery::TcpSocket queries because it sets the
+    local port number to \a bindPort. The specified \a networkConfiguration
+    is used to resolve the proxy settings.
+
+    Note that \a bindPort is of type quint16 to indicate the exact
+    port number that is requested. The value of -1 (unknown) is not
+    allowed in this context.
+
+    \sa localPort(), networkConfiguration()
+*/
+QNetworkProxyQuery::QNetworkProxyQuery(const QNetworkConfiguration &networkConfiguration,
+                                       quint16 bindPort, const QString &protocolTag,
+                                       QueryType queryType)
+{
+    d->config = networkConfiguration;
+    d->remote.setScheme(protocolTag);
+    d->localPort = bindPort;
+    d->type = queryType;
+}
+#endif
+
 /*!
     Constructs a QNetworkProxyQuery object that is a copy of \a other.
 */
@@ -1116,6 +1185,37 @@ void QNetworkProxyQuery::setUrl(const QUrl &url)
     d->remote = url;
 }
 
+#ifndef QT_NO_BEARERMANAGEMENT
+/*!
+    Returns the network configuration of the proxy query.
+
+    \sa setNetworkConfiguration()
+*/
+QNetworkConfiguration QNetworkProxyQuery::networkConfiguration() const
+{
+    return d ? d->config : QNetworkConfiguration();
+}
+
+/*!
+    \since 4.8
+
+    Sets the network configuration component of this QNetworkProxyQuery
+    object to be \a networkConfiguration. The network configuration can
+    be used to return different proxy settings based on the network in
+    use, for example WLAN vs cellular networks on a mobile phone.
+
+    In the case of "user choice" or "service network" configurations,
+    you should first start the QNetworkSession and obtain the active
+    configuration from its properties.
+
+    \sa networkConfiguration()
+*/
+void QNetworkProxyQuery::setNetworkConfiguration(const QNetworkConfiguration &networkConfiguration)
+{
+    d->config = networkConfiguration;
+}
+#endif
+
 /*!
     \class QNetworkProxyFactory
     \brief The QNetworkProxyFactory class provides fine-grained proxy selection.
@@ -1226,7 +1326,7 @@ void QNetworkProxyFactory::setApplicationProxyFactory(QNetworkProxyFactory *fact
 /*!
     \fn QList<QNetworkProxy> QNetworkProxyFactory::queryProxy(const QNetworkProxyQuery &query)
 
-    This function examines takes the query request, \a query,
+    This function takes the query request, \a query,
     examines the details of the type of socket or request and returns
     a list of QNetworkProxy objects that indicate the proxy servers to
     be used, in order of preference.
@@ -1247,7 +1347,7 @@ void QNetworkProxyFactory::setApplicationProxyFactory(QNetworkProxyFactory *fact
 /*!
     \fn QList<QNetworkProxy> QNetworkProxyFactory::systemProxyForQuery(const QNetworkProxyQuery &query)
 
-    This function examines takes the query request, \a query,
+    This function takes the query request, \a query,
     examines the details of the type of socket or request and returns
     a list of QNetworkProxy objects that indicate the proxy servers to
     be used, in order of preference.
@@ -1273,9 +1373,9 @@ void QNetworkProxyFactory::setApplicationProxyFactory(QNetworkProxyFactory *fact
     SOCKS server for all queries. If SOCKS isn't enabled, it will use
     the HTTPS proxy for all TcpSocket and UrlRequest queries.
 
-    On other systems, there is no standardised method of obtaining the
-    system proxy configuration. This function may be improved in
-    future versions to support those systems.
+    On other systems, this function will pick up proxy settings from
+    the "http_proxy" environment variable. This variable must be a URL
+    using one of the following schemes: "http", "socks5" or "socks5h".
 
     \section1 Limitations
 
@@ -1293,7 +1393,7 @@ void QNetworkProxyFactory::setApplicationProxyFactory(QNetworkProxyFactory *fact
 */
 
 /*!
-    This function examines takes the query request, \a query,
+    This function takes the query request, \a query,
     examines the details of the type of socket or request and returns
     a list of QNetworkProxy objects that indicate the proxy servers to
     be used, in order of preference.
