@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -40,6 +40,7 @@
 ****************************************************************************/
 
 #include "qgl_p.h"
+#include <qglframebufferobject.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -191,23 +192,30 @@ bool qt_resolve_frag_program_extensions(QGLContext *ctx)
 
 bool qt_resolve_buffer_extensions(QGLContext *ctx)
 {
+    if (!QGLContextPrivate::extensionFuncs(ctx).qt_bufferFuncsResolved) {
 #if defined(QGL_RESOLVE_BUFFER_FUNCS)
-    if (glBindBuffer && glDeleteBuffers && glGenBuffers && glBufferData
-            && glBufferSubData && glGetBufferParameteriv)
-        return true;
+        glBindBuffer = (_glBindBuffer) qt_gl_getProcAddressARB(ctx, "glBindBuffer");
+        glDeleteBuffers = (_glDeleteBuffers) qt_gl_getProcAddressARB(ctx, "glDeleteBuffers");
+        glGenBuffers = (_glGenBuffers) qt_gl_getProcAddressARB(ctx, "glGenBuffers");
+        glBufferData = (_glBufferData) qt_gl_getProcAddressARB(ctx, "glBufferData");
+        glBufferSubData = (_glBufferSubData) qt_gl_getProcAddressARB(ctx, "glBufferSubData");
+        glGetBufferSubData = (_glGetBufferSubData) qt_gl_getProcAddressARB(ctx, "glGetBufferSubData");
+        glGetBufferParameteriv = (_glGetBufferParameteriv) qt_gl_getProcAddressARB(ctx, "glGetBufferParameteriv");
 #endif
 
-#if defined(QGL_RESOLVE_BUFFER_FUNCS)
-    glBindBuffer = (_glBindBuffer) qt_gl_getProcAddressARB(ctx, "glBindBuffer");
-    glDeleteBuffers = (_glDeleteBuffers) qt_gl_getProcAddressARB(ctx, "glDeleteBuffers");
-    glGenBuffers = (_glGenBuffers) qt_gl_getProcAddressARB(ctx, "glGenBuffers");
-    glBufferData = (_glBufferData) qt_gl_getProcAddressARB(ctx, "glBufferData");
-    glBufferSubData = (_glBufferSubData) qt_gl_getProcAddressARB(ctx, "glBufferSubData");
-    glGetBufferSubData = (_glGetBufferSubData) qt_gl_getProcAddressARB(ctx, "glGetBufferSubData");
-    glGetBufferParameteriv = (_glGetBufferParameteriv) qt_gl_getProcAddressARB(ctx, "glGetBufferParameteriv");
+#ifdef QT_OPENGL_ES_2
+        QGLExtensionMatcher extensions;
+        if (extensions.match("GL_OES_mapbuffer")) {
+            glMapBufferARB = (_glMapBufferARB) qt_gl_getProcAddressARB(ctx, "glMapBufferOES");
+            glUnmapBufferARB = (_glUnmapBufferARB) qt_gl_getProcAddressARB(ctx, "glUnmapBufferOES");
+        }
+#else
+        glMapBufferARB = (_glMapBufferARB) qt_gl_getProcAddressARB(ctx, "glMapBuffer");
+        glUnmapBufferARB = (_glUnmapBufferARB) qt_gl_getProcAddressARB(ctx, "glUnmapBuffer");
 #endif
-    glMapBufferARB = (_glMapBufferARB) qt_gl_getProcAddressARB(ctx, "glMapBuffer");
-    glUnmapBufferARB = (_glUnmapBufferARB) qt_gl_getProcAddressARB(ctx, "glUnmapBuffer");
+
+        QGLContextPrivate::extensionFuncs(ctx).qt_bufferFuncsResolved = true;
+    }
 
 #if defined(QGL_RESOLVE_BUFFER_FUNCS)
     return glBindBuffer
@@ -411,9 +419,6 @@ bool qt_resolve_version_2_0_functions(QGLContext *ctx)
     if (!qt_resolve_version_1_3_functions(ctx))
         gl2supported = false;
 
-    if (!qt_resolve_framebufferobject_extensions(ctx))
-        gl2supported = false;
-
     if (glStencilOpSeparate)
         return gl2supported;
 
