@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -39,7 +39,7 @@
 **
 ****************************************************************************/
 
-#include <GLES/egltypes.h>
+#include <GLES/eglplatform.h>
 #include <wsegl.h>
 #include <pvr2d.h>
 #include <string.h>
@@ -202,7 +202,8 @@ static WSEGLError wseglDeleteDrawable(WSEGLDrawableHandle _drawable)
     PvrQwsDrawable *drawable = (PvrQwsDrawable *)_drawable;
     if (!drawable || drawable->type == PvrQwsScreen)
         return WSEGL_SUCCESS;
-    pvrQwsFreeBuffers(drawable);
+    if (pvrQwsDisplay.numFlipBuffers == 0)
+        pvrQwsFreeBuffers(drawable);
     if (pvrQwsReleaseWindow(drawable))
         pvrQwsDestroyDrawable(drawable);
     return WSEGL_SUCCESS;
