@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtSCriptTools module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -76,7 +76,7 @@ QT_BEGIN_NAMESPACE
 static QScriptValue debuggerResponseToScriptValue(QScriptEngine *eng, const QScriptDebuggerResponse &in)
 {
     QScriptValue out = eng->newObject();
-    out.setProperty(QString::fromLatin1("result"), qScriptValueFromValue(eng, in.result()));
+    out.setProperty(QString::fromLatin1("result"), eng->toScriptValue(in.result()));
     out.setProperty(QString::fromLatin1("error"), QScriptValue(eng, in.error()));
     out.setProperty(QString::fromLatin1("async"), QScriptValue(eng, in.async()));
     return out;
@@ -122,7 +122,7 @@ static QScriptValue breakpointMapToScriptValue(QScriptEngine *eng, const QScript
     QScriptValue out = eng->newObject();
     QScriptBreakpointMap::const_iterator it;
     for (it = in.constBegin(); it != in.constEnd(); ++it) {
-        out.setProperty(QString::number(it.key()), qScriptValueFromValue(eng, it.value()));
+        out.setProperty(QString::number(it.key()), eng->toScriptValue(it.value()));
     }
     return out;
 }
@@ -155,7 +155,7 @@ static QScriptValue scriptMapToScriptValue(QScriptEngine *eng, const QScriptScri
     QScriptValue out = eng->newObject();
     QScriptScriptMap::const_iterator it;
     for (it = in.constBegin(); it != in.constEnd(); ++it) {
-        out.setProperty(QString::number(it.key()), qScriptValueFromValue(eng, it.value()));
+        out.setProperty(QString::number(it.key()), eng->toScriptValue(it.value()));
     }
     return out;
 }
@@ -175,8 +175,8 @@ static QScriptValue consoleCommandToScriptValue(
     out.setProperty(QString::fromLatin1("group"), QScriptValue(eng, in->group()));
     out.setProperty(QString::fromLatin1("shortDescription"), QScriptValue(eng, in->shortDescription()));
     out.setProperty(QString::fromLatin1("longDescription"), QScriptValue(eng, in->longDescription()));
-    out.setProperty(QString::fromLatin1("aliases"), qScriptValueFromValue(eng, in->aliases()));
-    out.setProperty(QString::fromLatin1("seeAlso"), qScriptValueFromValue(eng, in->seeAlso()));
+    out.setProperty(QString::fromLatin1("aliases"), eng->toScriptValue(in->aliases()));
+    out.setProperty(QString::fromLatin1("seeAlso"), eng->toScriptValue(in->seeAlso()));
     return out;
 }
 
@@ -207,7 +207,7 @@ static QScriptValue consoleCommandGroupMapToScriptValue(
     QScriptValue out = eng->newObject();
     QScriptDebuggerConsoleCommandGroupMap::const_iterator it;
     for (it = in.constBegin(); it != in.constEnd(); ++it) {
-        out.setProperty(it.key(), qScriptValueFromValue(eng, it.value()));
+        out.setProperty(it.key(), eng->toScriptValue(it.value()));
     }
     return out;
 }
@@ -238,7 +238,7 @@ static QScriptValue debuggerScriptValuePropertyToScriptValue(QScriptEngine *eng,
 {
     QScriptValue out = eng->newObject();
     out.setProperty(QString::fromLatin1("name"), QScriptValue(eng, in.name()));
-    out.setProperty(QString::fromLatin1("value"), qScriptValueFromValue(eng, in.value()));
+    out.setProperty(QString::fromLatin1("value"), eng->toScriptValue(in.value()));
     out.setProperty(QString::fromLatin1("valueAsString"), QScriptValue(eng, in.valueAsString()));
     out.setProperty(QString::fromLatin1("flags"), QScriptValue(eng, static_cast<int>(in.flags())));
     return out;
